@@ -29,21 +29,10 @@
 #include <stdint.h>
 #include <stdio.h>
 
-#include "igt_display.h"
+enum pipe;
 
-/**
- * igt_debugfs_t:
- *
- * debugfs access structure. Needs to be initialized with igt_debugfs_init().
- */
-typedef struct {
-	char root[128];
-	char dri_path[128];
-} igt_debugfs_t;
-
-void igt_debugfs_init(igt_debugfs_t *debugfs);
-int igt_debugfs_open(igt_debugfs_t *debugfs, const char *filename, int mode);
-FILE *igt_debugfs_fopen(igt_debugfs_t *debugfs, const char *filename,
+int igt_debugfs_open(const char *filename, int mode);
+FILE *igt_debugfs_fopen(const char *filename,
 			const char *mode);
 
 /*
@@ -100,10 +89,9 @@ bool igt_crc_is_null(igt_crc_t *crc);
 bool igt_crc_equal(igt_crc_t *a, igt_crc_t *b);
 char *igt_crc_to_string(igt_crc_t *crc);
 
-void igt_pipe_crc_check(igt_debugfs_t *debugfs);
+void igt_require_pipe_crc(void);
 igt_pipe_crc_t *
-igt_pipe_crc_new(igt_debugfs_t *debugfs, enum pipe pipe,
-		 enum intel_pipe_crc_source source);
+igt_pipe_crc_new(enum pipe pipe, enum intel_pipe_crc_source source);
 void igt_pipe_crc_free(igt_pipe_crc_t *pipe_crc);
 void igt_pipe_crc_start(igt_pipe_crc_t *pipe_crc);
 void igt_pipe_crc_stop(igt_pipe_crc_t *pipe_crc);
@@ -154,5 +142,4 @@ void igt_drop_caches_set(uint64_t val);
 void igt_disable_prefault(void);
 void igt_enable_prefault(void);
 
-
 #endif /* __IGT_DEBUGFS_H__ */
