@@ -25,7 +25,7 @@
  * 	Damien Lespiau <damien.lespiau@intel.com>
  */
 
-#define _GNU_SOURCE
+#include "config.h"
 #include <unistd.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -33,8 +33,13 @@
 #include <sys/stat.h>
 #include <string.h>
 #include <stdlib.h>
+#ifdef HAVE_LINUX_KD_H
 #include <linux/kd.h>
+#elif HAVE_SYS_KD_H
+#include <sys/kd.h>
+#endif
 #include <errno.h>
+#include <time.h>
 
 #include <i915_drm.h>
 
@@ -42,6 +47,7 @@
 #include "igt_kms.h"
 #include "igt_aux.h"
 #include "intel_chipset.h"
+#include "igt_debugfs.h"
 
 /*
  * There hasn't been a release of libdrm containing these #define's yet, so
@@ -60,6 +66,114 @@
 static char *forced_connectors[MAX_CONNECTORS + 1];
 
 
+#define VFREQ 60
+#define CLOCK 148500
+#define HACTIVE 1920
+#define HBLANK 280
+#define VACTIVE 1080
+#define VBLANK 45
+#define HOFFSET 88
+#define HPULSE 44
+#define VOFFSET 4
+#define VPULSE 5
+
+#define HSIZE 52
+#define VSIZE 30
+
+#define GAMMA(x) (x * 100) - 100
+
+#define MANUFACTURER_ID(a, b, c) (a - '@') << 2 | (b - '@') >> 3, \
+				 (b - '@') << 5 | (c - '@')
+
+
+#define ab(x, y) (x & 0xff), (y & 0xff), ((x & 0xf00) >> 4) | ((y & 0xf00) >> 8)
+#define op(ho, hp, vo, vp) (ho & 0xff), (hp & 0xff), \
+			   ((vo & 0xf) << 4) | (vp & 0xf), \
+			   ((ho & 0x300) >> 2) | ((hp & 0x300) >> 4) \
+			   | ((vo & 0x30) >> 2) | (vp & 0x30 >> 4)
+
+static unsigned char base_edid[EDID_LENGTH] = {
+	0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, /* header */
+	MANUFACTURER_ID('I', 'G', 'T'),
+	/* product code, serial number, week and year of manufacture */
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x01, 0x03, /* edid version (1.3) */
+	/* basic display parameters */
+	/* digital display, maximum horizontal image size, maximum vertical
+	 * image size, gamma, features: RGB 4:4:4, native pixel format and
+	 * refresh rate in descriptor 1 */
+	0x80, HSIZE, VSIZE, GAMMA(2.20), 0x02,
+	/* chromaticity coordinates */
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	/* established timings: 640x480 60Hz, 800x600 60Hz, 1024x768 60Hz */
+	0x21, 0x08, 0x00,
+	/* standard timings */
+	0xd1, 0xc0, /* 1920x1080 60Hz */
+	0x81, 0xc0, /* 1280x720 60Hz */
+	0x61, 0x40, /* 1024x768 60Hz */
+	0x45, 0x40, /* 800x600 60Hz */
+	0x31, 0x40, /* 640x480 60Hz */
+	0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
+	/* descriptor 1 (preferred timing) */
+	(CLOCK / 10) & 0x00ff, ((CLOCK / 10) & 0xff00) >> 8,
+	ab(HACTIVE, HBLANK), ab(VACTIVE, VBLANK),
+	op(HOFFSET, HPULSE, VOFFSET, VPULSE),
+	ab(HSIZE * 10, VSIZE * 10),
+	0x00, 0x00, 0x00,
+	/* descriptor 2 (monitor range limits) */
+	0x00, 0x00, 0x00, 0xfd, 0x00,
+	VFREQ - 1, VFREQ + 1, /* minimum, maximum vertical field rate */
+	(CLOCK / (HACTIVE + HBLANK)) - 1, /* minimum horizontal line rate */
+	(CLOCK / (HACTIVE + HBLANK)) + 1, /* maximum horizontal line rate */
+	(CLOCK / 10000) + 1, /* maximum pixel clock rate */
+	0x00, 0x0a, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
+	/* descriptor 3 (name descriptor) */
+	0x00, 0x00, 0x00, 0xfc, 0x00,  'I',  'G',  'T', 0x0a,
+	0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
+	/* descriptor 4 */
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+	/* extensions, checksum */
+	0x00, 0x00
+};
+
+/**
+ * igt_kms_get_base_edid:
+ *
+ * Get the base edid block, which includes the following modes:
+ *
+ *  - 1920x1080 60Hz
+ *  - 1280x720 60Hz
+ *  - 1024x768 60Hz
+ *  - 800x600 60Hz
+ *  - 640x480 60Hz
+ *
+ * This can be extended with further features using functions such as
+ * #kmstest_edid_add_3d.
+ *
+ * Returns: a basic edid block
+ */
+const unsigned char* igt_kms_get_base_edid(void)
+{
+	int i, sum = 0;
+	struct tm *tm;
+	time_t t;
+
+	/* year of manufacture */
+	t = time(NULL);
+	tm = localtime(&t);
+	base_edid[17] = tm->tm_year - 90;
+
+	/* calculate checksum */
+	for (i = 0; i < 127; i++) {
+		sum = sum + base_edid[i];
+	}
+	base_edid[127] = 256 - sum;
+
+	return base_edid;
+}
+
+
 /**
  * SECTION:igt_kms
  * @short_description: Kernel modesetting support library
@@ -119,59 +233,6 @@ const char *kmstest_plane_name(enum igt_plane plane)
 	return names[plane];
 }
 
-struct type_name {
-	int type;
-	const char *name;
-};
-
-#define type_name_fn(res) \
-const char * kmstest_##res##_str(int type) {		\
-	unsigned int i;					\
-	for (i = 0; i < ARRAY_SIZE(res##_names); i++) { \
-		if (res##_names[i].type == type)	\
-			return res##_names[i].name;	\
-	}						\
-	return "(invalid)";				\
-}
-
-struct type_name encoder_type_names[] = {
-	{ DRM_MODE_ENCODER_NONE, "none" },
-	{ DRM_MODE_ENCODER_DAC, "DAC" },
-	{ DRM_MODE_ENCODER_TMDS, "TMDS" },
-	{ DRM_MODE_ENCODER_LVDS, "LVDS" },
-	{ DRM_MODE_ENCODER_TVDAC, "TVDAC" },
-};
-
-type_name_fn(encoder_type)
-
-struct type_name connector_status_names[] = {
-	{ DRM_MODE_CONNECTED, "connected" },
-	{ DRM_MODE_DISCONNECTED, "disconnected" },
-	{ DRM_MODE_UNKNOWNCONNECTION, "unknown" },
-};
-
-type_name_fn(connector_status)
-
-struct type_name connector_type_names[] = {
-	{ DRM_MODE_CONNECTOR_Unknown, "unknown" },
-	{ DRM_MODE_CONNECTOR_VGA, "VGA" },
-	{ DRM_MODE_CONNECTOR_DVII, "DVI-I" },
-	{ DRM_MODE_CONNECTOR_DVID, "DVI-D" },
-	{ DRM_MODE_CONNECTOR_DVIA, "DVI-A" },
-	{ DRM_MODE_CONNECTOR_Composite, "composite" },
-	{ DRM_MODE_CONNECTOR_SVIDEO, "s-video" },
-	{ DRM_MODE_CONNECTOR_LVDS, "LVDS" },
-	{ DRM_MODE_CONNECTOR_Component, "component" },
-	{ DRM_MODE_CONNECTOR_9PinDIN, "9-pin DIN" },
-	{ DRM_MODE_CONNECTOR_DisplayPort, "DP" },
-	{ DRM_MODE_CONNECTOR_HDMIA, "HDMI-A" },
-	{ DRM_MODE_CONNECTOR_HDMIB, "HDMI-B" },
-	{ DRM_MODE_CONNECTOR_TV, "TV" },
-	{ DRM_MODE_CONNECTOR_eDP, "eDP" },
-};
-
-type_name_fn(connector_type)
-
 static const char *mode_stereo_name(const drmModeModeInfo *mode)
 {
 	switch (mode->flags & DRM_MODE_FLAG_3D_MASK) {
@@ -238,12 +299,26 @@ int kmstest_get_pipe_from_crtc_id(int fd, int crtc_id)
 	return pfci.pipe;
 }
 
+/*
+ * Returns: the previous mode, or KD_GRAPHICS if no /dev/tty0 was
+ * found and nothing was done.
+ */
 static signed long set_vt_mode(unsigned long mode)
 {
 	int fd;
 	unsigned long prev_mode;
+	static const char TTY0[] = "/dev/tty0";
 
-	fd = open("/dev/tty0", O_RDONLY);
+	if (access(TTY0, F_OK)) {
+		/* errno message should be "No such file". Do not
+		   hardcode but ask strerror() in the very unlikely
+		   case something else happened. */
+		igt_debug("VT: %s: %s, cannot change its mode\n",
+			  TTY0, strerror(errno));
+		return KD_GRAPHICS;
+	}
+
+	fd = open(TTY0, O_RDONLY);
 	if (fd < 0)
 		return -errno;
 
@@ -275,10 +350,10 @@ void kmstest_restore_vt_mode(void)
 
 	if (orig_vt_mode != -1UL) {
 		ret = set_vt_mode(orig_vt_mode);
-		orig_vt_mode = -1UL;
 
 		igt_assert(ret >= 0);
-		igt_debug("VT: original mode restored\n");
+		igt_debug("VT: original mode 0x%lx restored\n", orig_vt_mode);
+		orig_vt_mode = -1UL;
 	}
 }
 
@@ -305,31 +380,13 @@ void kmstest_set_vt_graphics_mode(void)
 	igt_assert(ret >= 0);
 	orig_vt_mode = ret;
 
-	igt_debug("VT: graphics mode set\n");
+	igt_debug("VT: graphics mode set (mode was 0x%lx)\n", ret);
 }
 
-static int get_card_number(int fd)
+
+static void reset_connectors_at_exit(int sig)
 {
-	struct stat buf;
-
-	/* find the minor number of the device */
-	fstat(fd, &buf);
-
-	return minor(buf.st_rdev) & 0x3f;
-}
-
-static char* get_debugfs_connector_path(int drm_fd, drmModeConnector *connector,
-					const char *file)
-{
-	char *path;
-
-	asprintf(&path, "/sys/kernel/debug/dri/%d/%s-%d/%s",
-		 get_card_number(drm_fd),
-		 kmstest_connector_type_str(connector->connector_type),
-		 connector->connector_type_id,
-		 file);
-
-	return path;
+	igt_reset_connectors();
 }
 
 /**
@@ -377,8 +434,10 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
 		break;
 	}
 
-	path = get_debugfs_connector_path(drm_fd, connector, "force");
-	debugfs_fd = open(path, O_WRONLY | O_TRUNC);
+	asprintf(&path, "%s-%d/force",
+		 kmstest_connector_type_str(connector->connector_type),
+		 connector->connector_type_id);
+	debugfs_fd = igt_debugfs_open(path, O_WRONLY | O_TRUNC);
 
 	if (debugfs_fd == -1) {
 		return false;
@@ -411,6 +470,7 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
 		tmp++;
 	}
 
+	igt_install_exit_handler(reset_connectors_at_exit);
 
 	igt_assert(ret != -1);
 	return (ret == -1) ? false : true;
@@ -424,8 +484,7 @@ bool kmstest_force_connector(int drm_fd, drmModeConnector *connector,
  * @length: length of the EDID data. #EDID_LENGTH defines the standard EDID
  * length
  *
- * Set the EDID data on @connector to @edid. See #generic_edid and
- * #kmstest_generic_edid for a set of generic EDID data blocks.
+ * Set the EDID data on @connector to @edid. See also #igt_kms_get_base_edid.
  *
  * If @length is zero, the forced EDID will be removed.
  */
@@ -435,10 +494,10 @@ void kmstest_force_edid(int drm_fd, drmModeConnector *connector,
 	char *path;
 	int debugfs_fd, ret;
 
-	path = get_debugfs_connector_path(drm_fd, connector, "edid_override");
-
-	debugfs_fd = open(path, O_WRONLY | O_TRUNC);
-
+	asprintf(&path, "%s-%d/edid_override",
+		 kmstest_connector_type_str(connector->connector_type),
+		 connector->connector_type_id);
+	debugfs_fd = igt_debugfs_open(path, O_WRONLY | O_TRUNC);
 	free(path);
 
 	igt_assert(debugfs_fd != -1);
@@ -814,7 +873,7 @@ igt_display_log(igt_display_t *display, const char *fmt, ...)
 	igt_debug("display: ");
 	for (i = 0; i < display->log_shift; i++)
 		igt_debug("%s", LOG_SPACES);
-	igt_vlog(IGT_LOG_DEBUG, fmt, args);
+	igt_vlog(IGT_LOG_DOMAIN, IGT_LOG_DEBUG, fmt, args);
 	va_end(args);
 }
 
@@ -1269,7 +1328,7 @@ static int igt_drm_plane_commit(igt_plane_t *plane,
 				      fb_id,
 				      0,    /* flags */
 				      plane->crtc_x, plane->crtc_y,
-				      plane->fb->width, plane->fb->height,
+				      plane->crtc_w, plane->crtc_h,
 				      IGT_FIXED(0,0), /* src_x */
 				      IGT_FIXED(0,0), /* src_y */
 				      IGT_FIXED(plane->fb->width,0), /* src_w */
@@ -1314,12 +1373,12 @@ static int igt_cursor_commit_legacy(igt_plane_t *cursor,
 			    igt_output_name(output),
 			    kmstest_pipe_name(output->config.pipe),
 			    gem_handle,
-			    cursor->fb->width, cursor->fb->height);
+			    cursor->crtc_w, cursor->crtc_h);
 
 			ret = drmModeSetCursor(display->drm_fd, crtc_id,
 					       gem_handle,
-					       cursor->fb->width,
-					       cursor->fb->height);
+					       cursor->crtc_w,
+					       cursor->crtc_h);
 		} else {
 			LOG(display,
 			    "%s: SetCursor pipe %s, disabling\n",
@@ -1631,6 +1690,14 @@ void igt_plane_set_fb(igt_plane_t *plane, struct igt_fb *fb)
 	    plane->index, fb ? fb->fb_id : 0);
 
 	plane->fb = fb;
+	/* hack to keep tests working that don't call igt_plane_set_size() */
+	if (fb) {
+		plane->crtc_w = fb->width;
+		plane->crtc_h = fb->height;
+	} else {
+		plane->crtc_w = 0;
+		plane->crtc_h = 0;
+	}
 
 	plane->fb_changed = true;
 }
@@ -1649,6 +1716,24 @@ void igt_plane_set_position(igt_plane_t *plane, int x, int y)
 	plane->position_changed = true;
 }
 
+void igt_plane_set_size(igt_plane_t *plane, int w, int h)
+{
+	igt_pipe_t *pipe = plane->pipe;
+	igt_display_t *display = pipe->display;
+
+	LOG(display, "%s.%d: plane_set_size(%d,%d)\n",
+	    kmstest_pipe_name(pipe->pipe), plane->index, w, h);
+
+	plane->crtc_w = w;
+	plane->crtc_h = h;
+
+	/*
+	 * must be fb_changed so that legacy cursors call
+	 * drmModeSetCursor() instead of drmModeMoveCursor()
+	 */
+	plane->fb_changed = true;
+}
+
 void igt_plane_set_panning(igt_plane_t *plane, int x, int y)
 {
 	igt_pipe_t *pipe = plane->pipe;
@@ -1707,11 +1792,6 @@ void igt_wait_for_vblank(int drm_fd, enum pipe pipe)
 	igt_assert(drmWaitVBlank(drm_fd, &wait_vbl) == 0);
 }
 
-static void reset_connectors_at_exit(int sig)
-{
-	igt_reset_connectors();
-}
-
 /**
  * igt_enable_connectors:
  *
@@ -1751,8 +1831,6 @@ void igt_enable_connectors(void)
 		drmModeFreeConnector(c);
 	}
 	close(drm_fd);
-
-	igt_install_exit_handler(reset_connectors_at_exit);
 }
 
 /**
@@ -1768,8 +1846,8 @@ void igt_reset_connectors(void)
 	 * functions that are not safe to call in signal handlers */
 
 	for (tmp = forced_connectors; *tmp; tmp++) {
-		int fd = open(*tmp, O_WRONLY | O_TRUNC);
-		write(fd, "unspecified", 11);
+		int fd = igt_debugfs_open(*tmp, O_WRONLY | O_TRUNC);
+		igt_assert(write(fd, "unspecified", 11) == 11);
 		close(fd);
 	}
 }
