@@ -54,7 +54,6 @@
 #include "intel_chipset.h"
 #include "intel_io.h"
 #include "igt_debugfs.h"
-#include "version.h"
 #include "config.h"
 
 #include "ioctl_wrappers.h"
@@ -97,6 +96,7 @@ gem_handle_to_libdrm_bo(drm_intel_bufmgr *bufmgr, int fd, const char *name, uint
 	int ret;
 	drm_intel_bo *bo;
 
+	memset(&flink, 0, sizeof(handle));
 	flink.handle = handle;
 	ret = ioctl(fd, DRM_IOCTL_GEM_FLINK, &flink);
 	igt_assert(ret == 0);
@@ -108,6 +108,31 @@ gem_handle_to_libdrm_bo(drm_intel_bufmgr *bufmgr, int fd, const char *name, uint
 	return bo;
 }
 
+/**
+ * gem_get_tiling:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @tiling: (out) tiling mode of the gem buffer
+ * @swizzle: (out) bit 6 swizzle mode
+ *
+ * This wraps the GET_TILING ioctl.
+ */
+void
+gem_get_tiling(int fd, uint32_t handle, uint32_t *tiling, uint32_t *swizzle)
+{
+	struct drm_i915_gem_get_tiling get_tiling;
+	int ret;
+
+	memset(&get_tiling, 0, sizeof(get_tiling));
+	get_tiling.handle = handle;
+
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_GET_TILING, &get_tiling);
+	igt_assert(ret == 0);
+
+	*tiling = get_tiling.tiling_mode;
+	*swizzle = get_tiling.swizzle_mode;
+}
+
 int __gem_set_tiling(int fd, uint32_t handle, uint32_t tiling, uint32_t stride)
 {
 	struct drm_i915_gem_set_tiling st;
@@ -171,6 +196,7 @@ void gem_set_caching(int fd, uint32_t handle, uint32_t caching)
 	struct local_drm_i915_gem_caching arg;
 	int ret;
 
+	memset(&arg, 0, sizeof(arg));
 	arg.handle = handle;
 	arg.caching = caching;
 	ret = ioctl(fd, LOCAL_DRM_IOCTL_I915_GEM_SET_CACHEING, &arg);
@@ -217,6 +243,7 @@ uint32_t gem_open(int fd, uint32_t name)
 	struct drm_gem_open open_struct;
 	int ret;
 
+	memset(&open_struct, 0, sizeof(open_struct));
 	open_struct.name = name;
 	ret = ioctl(fd, DRM_IOCTL_GEM_OPEN, &open_struct);
 	igt_assert(ret == 0);
@@ -242,6 +269,7 @@ uint32_t gem_flink(int fd, uint32_t handle)
 	struct drm_gem_flink flink;
 	int ret;
 
+	memset(&flink, 0, sizeof(flink));
 	flink.handle = handle;
 	ret = ioctl(fd, DRM_IOCTL_GEM_FLINK, &flink);
 	igt_assert(ret == 0);
@@ -262,6 +290,7 @@ void gem_close(int fd, uint32_t handle)
 {
 	struct drm_gem_close close_bo;
 
+	memset(&close_bo, 0, sizeof(close_bo));
 	close_bo.handle = handle;
 	do_ioctl(fd, DRM_IOCTL_GEM_CLOSE, &close_bo);
 }
@@ -281,6 +310,7 @@ void gem_write(int fd, uint32_t handle, uint32_t offset, const void *buf, uint32
 {
 	struct drm_i915_gem_pwrite gem_pwrite;
 
+	memset(&gem_pwrite, 0, sizeof(gem_pwrite));
 	gem_pwrite.handle = handle;
 	gem_pwrite.offset = offset;
 	gem_pwrite.size = length;
@@ -303,6 +333,7 @@ void gem_read(int fd, uint32_t handle, uint32_t offset, void *buf, uint32_t leng
 {
 	struct drm_i915_gem_pread gem_pread;
 
+	memset(&gem_pread, 0, sizeof(gem_pread));
 	gem_pread.handle = handle;
 	gem_pread.offset = offset;
 	gem_pread.size = length;
@@ -327,6 +358,7 @@ void gem_set_domain(int fd, uint32_t handle,
 {
 	struct drm_i915_gem_set_domain set_domain;
 
+	memset(&set_domain, 0, sizeof(set_domain));
 	set_domain.handle = handle;
 	set_domain.read_domains = read_domains;
 	set_domain.write_domain = write_domain;
@@ -352,6 +384,7 @@ uint32_t __gem_create(int fd, int size)
 	struct drm_i915_gem_create create;
 	int ret;
 
+	memset(&create, 0, sizeof(create));
 	create.handle = 0;
 	create.size = size;
 	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
@@ -377,6 +410,7 @@ uint32_t gem_create(int fd, int size)
 {
 	struct drm_i915_gem_create create;
 
+	memset(&create, 0, sizeof(create));
 	create.handle = 0;
 	create.size = size;
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_CREATE, &create);
@@ -421,6 +455,7 @@ void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
 	struct drm_i915_gem_mmap_gtt mmap_arg;
 	void *ptr;
 
+	memset(&mmap_arg, 0, sizeof(mmap_arg));
 	mmap_arg.handle = handle;
 	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP_GTT, &mmap_arg))
 		return NULL;
@@ -428,15 +463,99 @@ void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
 	ptr = mmap64(0, size, prot, MAP_SHARED, fd, mmap_arg.offset);
 	if (ptr == MAP_FAILED)
 		ptr = NULL;
+	else
+		errno = 0;
 
 	return ptr;
 }
 
+struct local_i915_gem_mmap_v2 {
+	uint32_t handle;
+	uint32_t pad;
+	uint64_t offset;
+	uint64_t size;
+	uint64_t addr_ptr;
+	uint64_t flags;
+#define I915_MMAP_WC 0x1
+};
+#define LOCAL_IOCTL_I915_GEM_MMAP_v2 DRM_IOWR(DRM_COMMAND_BASE + DRM_I915_GEM_MMAP, struct local_i915_gem_mmap_v2)
+
+bool gem_mmap__has_wc(int fd)
+{
+	static int has_wc = -1;
+
+	if (has_wc == -1) {
+		struct drm_i915_getparam gp;
+		int val = -1;
+
+		has_wc = 0;
+
+		memset(&gp, 0, sizeof(gp));
+		gp.param = 30; /* MMAP_VERSION */
+		gp.value = &val;
+
+		/* Do we have the new mmap_ioctl? */
+		ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp);
+		if (val >= 1) {
+			struct local_i915_gem_mmap_v2 arg;
+
+			/* Does this device support wc-mmaps ? */
+			memset(&arg, 0, sizeof(arg));
+			arg.handle = gem_create(fd, 4096);
+			arg.offset = 0;
+			arg.size = 4096;
+			arg.flags = I915_MMAP_WC;
+			has_wc = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg) == 0;
+			gem_close(fd, arg.handle);
+		}
+		errno = 0;
+	}
+
+	return has_wc > 0;
+}
+
+/**
+ * gem_mmap__wc:
+ * @fd: open i915 drm file descriptor
+ * @handle: gem buffer object handle
+ * @offset: offset in the gem buffer of te mmap arena
+ * @size: size of the mmap arena
+ * @prot: memory protection bits as used by mmap()
+ *
+ * This functions wraps up procedure to establish a memory mapping through
+ * direct cpu access, bypassing the gpu and cpu caches completely and also
+ * bypassing the GTT system agent (i.e. there is no automatic tiling of
+ * the mmapping through the fence registers).
+ *
+ * Returns: A pointer to the created memory mapping.
+ */
+void *gem_mmap__wc(int fd, uint32_t handle, int offset, int size, int prot)
+{
+	struct local_i915_gem_mmap_v2 arg;
+
+	if (!gem_mmap__has_wc(fd)) {
+		errno = ENOSYS;
+		return NULL;
+	}
+
+	memset(&arg, 0, sizeof(arg));
+	arg.handle = handle;
+	arg.offset = offset;
+	arg.size = size;
+	arg.flags = I915_MMAP_WC;
+	if (drmIoctl(fd, LOCAL_IOCTL_I915_GEM_MMAP_v2, &arg))
+		return NULL;
+
+	errno = 0;
+	return (void *)(uintptr_t)arg.addr_ptr;
+}
+
 /**
  * gem_mmap__cpu:
  * @fd: open i915 drm file descriptor
  * @handle: gem buffer object handle
- * @size: size of the gem buffer
+ * @offset: offset in the gem buffer of te mmap arena
+ * @size: size of the mmap arena
  * @prot: memory protection bits as used by mmap()
  *
  * This functions wraps up procedure to establish a memory mapping through
@@ -444,12 +563,13 @@ void *gem_mmap__gtt(int fd, uint32_t handle, int size, int prot)
  *
  * Returns: A pointer to the created memory mapping.
  */
-void *gem_mmap__cpu(int fd, uint32_t handle, int size, int prot)
+void *gem_mmap__cpu(int fd, uint32_t handle, int offset, int size, int prot)
 {
 	struct drm_i915_gem_mmap mmap_arg;
 
+	memset(&mmap_arg, 0, sizeof(mmap_arg));
 	mmap_arg.handle = handle;
-	mmap_arg.offset = 0;
+	mmap_arg.offset = offset;
 	mmap_arg.size = size;
 	if (drmIoctl(fd, DRM_IOCTL_I915_GEM_MMAP, &mmap_arg))
 		return NULL;
@@ -470,12 +590,13 @@ void *gem_mmap__cpu(int fd, uint32_t handle, int size, int prot)
  * object is in need again it must be set back to WILLNEED before first use.
  *
  * Returns: When setting the madvise state to WILLNEED this returns whether the
- * backing storage was still avialable or not.
+ * backing storage was still available or not.
  */
 int gem_madvise(int fd, uint32_t handle, int state)
 {
 	struct drm_i915_gem_madvise madv;
 
+	memset(&madv, 0, sizeof(madv));
 	madv.handle = handle;
 	madv.madv = state;
 	madv.retained = 1;
@@ -500,6 +621,7 @@ uint32_t gem_context_create(int fd)
 	struct drm_i915_gem_context_create create;
 	int ret;
 
+	memset(&create, 0, sizeof(create));
 	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create);
 	igt_require(ret == 0 || (errno != ENODEV && errno != EINVAL));
 	igt_assert(ret == 0);
@@ -508,6 +630,92 @@ uint32_t gem_context_create(int fd)
 	return create.ctx_id;
 }
 
+int __gem_context_destroy(int fd, uint32_t ctx_id)
+{
+	struct drm_i915_gem_context_destroy destroy;
+	int ret;
+
+	memset(&destroy, 0, sizeof(destroy));
+	destroy.ctx_id = ctx_id;
+
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
+	if (ret)
+		return -errno;
+	return 0;
+}
+
+/**
+ * gem_context_destroy:
+ * @fd: open i915 drm file descriptor
+ * @ctx_id: i915 hw context id
+ *
+ * This is a wraps the CONTEXT_DESTROY ioctl, which is used to free a hardware
+ * context.
+ */
+void gem_context_destroy(int fd, uint32_t ctx_id)
+{
+	struct drm_i915_gem_context_destroy destroy;
+
+	memset(&destroy, 0, sizeof(destroy));
+	destroy.ctx_id = ctx_id;
+
+	do_ioctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_DESTROY, &destroy);
+}
+
+/**
+ * gem_context_get_param:
+ * @fd: open i915 drm file descriptor
+ * @p: i915 hw context parameter
+ *
+ * This is a wraps the CONTEXT_GET_PARAM ioctl, which is used to free a hardware
+ * context. Not that similarly to gem_set_caching() this wrapper calls
+ * igt_require() internally to correctly skip on kernels and platforms where hw
+ * context parameter support is not available.
+ */
+void gem_context_get_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_GETPARAM       0x34
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_GETPARAM, struct local_i915_gem_context_param)
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, p);
+}
+
+/**
+ * gem_context_set_param:
+ * @fd: open i915 drm file descriptor
+ * @p: i915 hw context parameter
+ *
+ * This is a wraps the CONTEXT_SET_PARAM ioctl, which is used to free a hardware
+ * context. Not that similarly to gem_set_caching() this wrapper calls
+ * igt_require() internally to correctly skip on kernels and platforms where hw
+ * context parameter support is not available.
+ */
+void gem_context_set_param(int fd, struct local_i915_gem_context_param *p)
+{
+#define LOCAL_I915_GEM_CONTEXT_SETPARAM       0x35
+#define LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM DRM_IOWR (DRM_COMMAND_BASE + LOCAL_I915_GEM_CONTEXT_SETPARAM, struct local_i915_gem_context_param)
+	do_ioctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_SETPARAM, p);
+}
+
+/**
+ * gem_context_require_param:
+ * @fd: open i915 drm file descriptor
+ * @param: i915 hw context parameter
+ *
+ * Feature test macro to query whether hw context parameter support for @param
+ * is available. Automatically skips through igt_require() if not.
+ */
+void gem_context_require_param(int fd, uint64_t param)
+{
+	struct local_i915_gem_context_param p;
+
+	p.context = 0;
+	p.param = param;
+	p.value = 0;
+	p.size = 0;
+
+	igt_require(drmIoctl(fd, LOCAL_IOCTL_I915_GEM_CONTEXT_GETPARAM, &p) == 0);
+}
+
 /**
  * gem_sw_finish:
  * @fd: open i915 drm file descriptor
@@ -521,6 +729,7 @@ void gem_sw_finish(int fd, uint32_t handle)
 {
 	struct drm_i915_gem_sw_finish finish;
 
+	memset(&finish, 0, sizeof(finish));
 	finish.handle = handle;
 
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_SW_FINISH, &finish);
@@ -540,6 +749,7 @@ bool gem_bo_busy(int fd, uint32_t handle)
 {
 	struct drm_i915_gem_busy busy;
 
+	memset(&busy, 0, sizeof(busy));
 	busy.handle = handle;
 
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_BUSY, &busy);
@@ -557,7 +767,7 @@ bool gem_bo_busy(int fd, uint32_t handle)
  * Feature test macro to check whether the kernel internally uses ppgtt to
  * execute batches. The /aliasing/ in the function name is a bit a misnomer,
  * this driver parameter is also true when full ppgtt address spaces are
- * availabel since for batchbuffer construction only ppgtt or global gtt is
+ * available since for batchbuffer construction only ppgtt or global gtt is
  * relevant.
  *
  * Returns: Whether batches are run through ppgtt.
@@ -565,8 +775,9 @@ bool gem_bo_busy(int fd, uint32_t handle)
 bool gem_uses_aliasing_ppgtt(int fd)
 {
 	struct drm_i915_getparam gp;
-	int val;
+	int val = 0;
 
+	memset(&gp, 0, sizeof(gp));
 	gp.param = 18; /* HAS_ALIASING_PPGTT */
 	gp.value = &val;
 
@@ -578,19 +789,20 @@ bool gem_uses_aliasing_ppgtt(int fd)
 }
 
 /**
- * gem_uses_aliasing_ppgtt:
+ * gem_available_fences:
  * @fd: open i915 drm file descriptor
  *
  * Feature test macro to query the kernel for the number of available fences
- * useable in a batchbuffer. Only relevant for pre-gen4.
+ * usable in a batchbuffer. Only relevant for pre-gen4.
  *
  * Returns: The number of available fences.
  */
 int gem_available_fences(int fd)
 {
 	struct drm_i915_getparam gp;
-	int val;
+	int val = 0;
 
+	memset(&gp, 0, sizeof(gp));
 	gp.param = I915_PARAM_NUM_FENCES_AVAIL;
 	gp.value = &val;
 
@@ -601,11 +813,27 @@ int gem_available_fences(int fd)
 	return val;
 }
 
+bool gem_has_llc(int fd)
+{
+	struct drm_i915_getparam gp;
+	int val = 0;
+
+	memset(&gp, 0, sizeof(gp));
+	gp.param = I915_PARAM_HAS_LLC;
+	gp.value = &val;
+
+	if (ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp)))
+		return 0;
+
+	errno = 0;
+	return val;
+}
+
 /**
  * gem_get_num_rings:
  * @fd: open i915 drm file descriptor
  *
- * Feature test macro to query the number of avaible rings. This is useful in
+ * Feature test macro to query the number of available rings. This is useful in
  * test loops which need to step through all rings and similar logic.
  *
  * For more explicit tests of ring availability see gem_has_enable_ring() and
@@ -644,14 +872,14 @@ skip:
  *
  * Feature test macro to query whether a specific ring is available.
  *
- * Returns: Whether the ring is avaible or not.
+ * Returns: Whether the ring is available or not.
  */
 bool gem_has_enable_ring(int fd,int param)
 {
 	drm_i915_getparam_t gp;
-	int tmp;
-	memset(&gp, 0, sizeof(gp));
+	int tmp = 0;
 
+	memset(&gp, 0, sizeof(gp));
 	gp.value = &tmp;
 	gp.param = param;
 
@@ -671,7 +899,7 @@ bool gem_has_enable_ring(int fd,int param)
  *
  * Note that recent Bspec calls this the VCS ring for Video Command Submission.
  *
- * Returns: Whether the BSD ring is avaible or not.
+ * Returns: Whether the BSD ring is available or not.
  */
 bool gem_has_bsd(int fd)
 {
@@ -687,11 +915,10 @@ bool gem_has_bsd(int fd)
  *
  * Note that recent Bspec calls this the BCS ring for Blitter Command Submission.
  *
- * Returns: Whether the blitter ring is avaible or not.
+ * Returns: Whether the blitter ring is available or not.
  */
 bool gem_has_blt(int fd)
 {
-
 	return gem_has_enable_ring(fd,I915_PARAM_HAS_BLT);
 }
 
@@ -706,20 +933,35 @@ bool gem_has_blt(int fd)
  * Note that recent Bspec calls this the VECS ring for Video Enhancement Command
  * Submission.
  *
- * Returns: Whether the vebox ring is avaible or not.
+ * Returns: Whether the vebox ring is available or not.
  */
 bool gem_has_vebox(int fd)
 {
-
 	return gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_VEBOX);
 }
 
+#define LOCAL_I915_PARAM_HAS_BSD2 31
+/**
+ * gem_has_bsd2:
+ * @fd: open i915 drm file descriptor
+ *
+ * Feature test macro to query whether the BSD2 ring is available. This is simply
+ * a specific version of gem_has_enable_ring() for the BSD2 ring.
+ *
+ * Note that recent Bspec calls this the VCS ring for Video Command Submission.
+ *
+ * Returns: Whether the BSD ring is avaible or not.
+ */
+bool gem_has_bsd2(int fd)
+{
+	return gem_has_enable_ring(fd,LOCAL_I915_PARAM_HAS_BSD2);
+}
 /**
  * gem_available_aperture_size:
  * @fd: open i915 drm file descriptor
  *
  * Feature test macro to query the kernel for the available gpu aperture size
- * useable in a batchbuffer.
+ * usable in a batchbuffer.
  *
  * Returns: The available gtt address space size.
  */
@@ -727,8 +969,10 @@ uint64_t gem_available_aperture_size(int fd)
 {
 	struct drm_i915_gem_get_aperture aperture;
 
+	memset(&aperture, 0, sizeof(aperture));
 	aperture.aper_size = 256*1024*1024;
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
 	return aperture.aper_available_size;
 }
 
@@ -744,25 +988,25 @@ uint64_t gem_aperture_size(int fd)
 {
 	struct drm_i915_gem_get_aperture aperture;
 
+	memset(&aperture, 0, sizeof(aperture));
 	aperture.aper_size = 256*1024*1024;
 	do_ioctl(fd, DRM_IOCTL_I915_GEM_GET_APERTURE, &aperture);
+
 	return aperture.aper_size;
 }
 
 /**
- * gem_aperture_size:
- * @fd: open i915 drm file descriptor
+ * gem_mappable_aperture_size:
  *
  * Feature test macro to query the kernel for the mappable gpu aperture size.
- * This is the area avaialble for GTT memory mappings.
+ * This is the area available for GTT memory mappings.
  *
  * Returns: The mappable gtt address space size.
  */
 uint64_t gem_mappable_aperture_size(void)
 {
-	struct pci_device *pci_dev;
+	struct pci_device *pci_dev = intel_get_pci_device();
 	int bar;
-	pci_dev = intel_get_pci_device();
 
 	if (intel_gen(pci_dev->device_id) < 3)
 		bar = 0;
@@ -784,6 +1028,7 @@ void gem_require_caching(int fd)
 	struct local_drm_i915_gem_caching arg;
 	int ret;
 
+	memset(&arg, 0, sizeof(arg));
 	arg.handle = gem_create(fd, 4096);
 	igt_assert(arg.handle != 0);
 
@@ -843,6 +1088,7 @@ int prime_handle_to_fd(int fd, uint32_t handle)
 {
 	struct drm_prime_handle args;
 
+	memset(&args, 0, sizeof(args));
 	args.handle = handle;
 	args.flags = DRM_CLOEXEC;
 	args.fd = -1;
@@ -866,6 +1112,7 @@ uint32_t prime_fd_to_handle(int fd, int dma_buf_fd)
 {
 	struct drm_prime_handle args;
 
+	memset(&args, 0, sizeof(args));
 	args.fd = dma_buf_fd;
 	args.flags = 0;
 	args.handle = 0;
@@ -888,6 +1135,7 @@ uint32_t prime_fd_to_handle(int fd, int dma_buf_fd)
 off_t prime_get_size(int dma_buf_fd)
 {
 	off_t ret;
+
 	ret = lseek(dma_buf_fd, 0, SEEK_END);
 	igt_assert(ret >= 0 || errno == ESPIPE);
 	igt_require(ret >= 0);
@@ -896,3 +1144,51 @@ off_t prime_get_size(int dma_buf_fd)
 	return ret;
 }
 
+/**
+ * igt_require_fb_modifiers:
+ * @fd: Open DRM file descriptor.
+ *
+ * Requires presence of DRM_CAP_ADDFB2_MODIFIERS.
+ */
+void igt_require_fb_modifiers(int fd)
+{
+	static bool has_modifiers, cap_modifiers_tested;
+
+	if (!cap_modifiers_tested) {
+		uint64_t cap_modifiers;
+		int ret;
+
+		ret = drmGetCap(fd, LOCAL_DRM_CAP_ADDFB2_MODIFIERS, &cap_modifiers);
+		igt_assert(ret == 0 || errno == EINVAL);
+		has_modifiers = ret == 0 && cap_modifiers == 1;
+		cap_modifiers_tested = true;
+	}
+
+	igt_require(has_modifiers);
+}
+
+int __kms_addfb(int fd, uint32_t handle, uint32_t width, uint32_t height,
+		uint32_t stride, uint32_t pixel_format, uint64_t modifier,
+		uint32_t flags, uint32_t *buf_id)
+{
+	struct local_drm_mode_fb_cmd2 f;
+	int ret;
+
+	igt_require_fb_modifiers(fd);
+
+	memset(&f, 0, sizeof(f));
+
+	f.width  = width;
+	f.height = height;
+	f.pixel_format = pixel_format;
+	f.flags = flags;
+	f.handles[0] = handle;
+	f.pitches[0] = stride;
+	f.modifier[0] = modifier;
+
+	ret = drmIoctl(fd, LOCAL_DRM_IOCTL_MODE_ADDFB2, &f);
+
+	*buf_id = f.fb_id;
+
+	return ret < 0 ? -errno : ret;
+}
