@@ -1,11 +1,29 @@
+#include <assert.h>
+#include <stdlib.h>
+#include <sys/ioctl.h>
+#include <stdio.h>
+#include <string.h>
+#include <assert.h>
+#include <fcntl.h>
+#include <inttypes.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/time.h>
+
+#include <drm.h>
+#include <i915_drm.h>
+
+#include "drmtest.h"
+#include "intel_bufmgr.h"
+#include "intel_batchbuffer.h"
+#include "intel_io.h"
 #include "rendercopy.h"
 #include "gen8_render.h"
-
-#include <assert.h>
+#include "intel_reg.h"
+#include "igt_aux.h"
 
 #include <intel_aub.h>
 
-#define ALIGN(x, y) (((x) + (y)-1) & ~((y)-1))
 #define VERTEX_SIZE (3*4)
 
 #if DEBUG_RENDERCPY
@@ -59,15 +77,15 @@ struct annotations_context {
 	drm_intel_aub_annotation annotations[MAX_ANNOTATIONS];
 	int index;
 	uint32_t offset;
-} aub_annotations;
+};
 
-static void annotation_init(struct annotations_context *ctx)
+static void annotation_init(struct annotations_context *aub)
 {
-	/* ctx->annotations is an array keeping a list of annotations of the
-	 * batch buffer ordered by offset. ctx->annotations[0] is thus left
+	/* aub->annotations is an array keeping a list of annotations of the
+	 * batch buffer ordered by offset. aub->annotations[0] is thus left
 	 * for the command stream and will be filled just before executing
 	 * the batch buffer with annotations_add_batch() */
-	ctx->index = 1;
+	aub->index = 1;
 }
 
 static void add_annotation(drm_intel_aub_annotation *a,
@@ -79,36 +97,36 @@ static void add_annotation(drm_intel_aub_annotation *a,
 	a->ending_offset = ending_offset;
 }
 
-static void annotation_add_batch(struct annotations_context *ctx, size_t size)
+static void annotation_add_batch(struct annotations_context *aub, size_t size)
 {
-	add_annotation(&ctx->annotations[0], AUB_TRACE_TYPE_BATCH, 0, size);
+	add_annotation(&aub->annotations[0], AUB_TRACE_TYPE_BATCH, 0, size);
 }
 
-static void annotation_add_state(struct annotations_context *ctx,
+static void annotation_add_state(struct annotations_context *aub,
 				 uint32_t state_type,
 				 uint32_t start_offset,
 				 size_t   size)
 {
-	assert(ctx->index < MAX_ANNOTATIONS);
+	igt_assert(aub->index < MAX_ANNOTATIONS);
 
-	add_annotation(&ctx->annotations[ctx->index++],
+	add_annotation(&aub->annotations[aub->index++],
 		       AUB_TRACE_TYPE_NOTYPE, 0,
 		       start_offset);
-	add_annotation(&ctx->annotations[ctx->index++],
+	add_annotation(&aub->annotations[aub->index++],
 		       AUB_TRACE_TYPE(state_type),
 		       AUB_TRACE_SUBTYPE(state_type),
 		       start_offset + size);
 }
 
-static void annotation_flush(struct annotations_context *ctx,
+static void annotation_flush(struct annotations_context *aub,
 			     struct intel_batchbuffer *batch)
 {
-	if (!drmtest_dump_aub())
+	if (!igt_aub_dump_enabled())
 		return;
 
 	drm_intel_bufmgr_gem_set_aub_annotations(batch->bo,
-						 ctx->annotations,
-						 ctx->index);
+						 aub->annotations,
+						 aub->index);
 }
 
 static uint32_t
@@ -156,13 +174,16 @@ gen6_render_flush(struct intel_batchbuffer *batch,
 	if (ret == 0)
 		ret = drm_intel_gem_bo_context_exec(batch->bo, context,
 						    batch_end, 0);
-	assert(ret == 0);
+	igt_assert(ret == 0);
 }
 
 /* Mostly copy+paste from gen6, except height, width, pitch moved */
 static uint32_t
-gen8_bind_buf(struct intel_batchbuffer *batch, struct scratch_buf *buf,
-	      uint32_t format, int is_dst) {
+gen8_bind_buf(struct intel_batchbuffer *batch,
+	      struct annotations_context *aub,
+	      struct igt_buf *buf,
+	      uint32_t format, int is_dst)
+{
 	struct gen8_surface_state *ss;
 	uint32_t write_domain, read_domain, offset;
 	int ret;
@@ -176,8 +197,7 @@ gen8_bind_buf(struct intel_batchbuffer *batch, struct scratch_buf *buf,
 
 	ss = batch_alloc(batch, sizeof(*ss), 64);
 	offset = batch_offset(batch, ss);
-	annotation_add_state(&aub_annotations, AUB_TRACE_SURFACE_STATE,
-			     offset, sizeof(*ss));
+	annotation_add_state(aub, AUB_TRACE_SURFACE_STATE, offset, sizeof(*ss));
 
 	ss->ss0.surface_type = GEN6_SURFACE_2D;
 	ss->ss0.surface_format = format;
@@ -195,10 +215,10 @@ gen8_bind_buf(struct intel_batchbuffer *batch, struct scratch_buf *buf,
 				      batch_offset(batch, ss) + 8 * 4,
 				      buf->bo, 0,
 				      read_domain, write_domain);
-	assert(ret == 0);
+	igt_assert(ret == 0);
 
-	ss->ss2.height = buf_height(buf) - 1;
-	ss->ss2.width  = buf_width(buf) - 1;
+	ss->ss2.height = igt_buf_height(buf) - 1;
+	ss->ss2.width  = igt_buf_width(buf) - 1;
 	ss->ss3.pitch  = buf->stride - 1;
 
 	ss->ss7.shader_chanel_select_r = 4;
@@ -211,33 +231,37 @@ gen8_bind_buf(struct intel_batchbuffer *batch, struct scratch_buf *buf,
 
 static uint32_t
 gen8_bind_surfaces(struct intel_batchbuffer *batch,
-		   struct scratch_buf *src,
-		   struct scratch_buf *dst)
+		   struct annotations_context *aub,
+		   struct igt_buf *src,
+		   struct igt_buf *dst)
 {
 	uint32_t *binding_table, offset;
 
 	binding_table = batch_alloc(batch, 8, 32);
 	offset = batch_offset(batch, binding_table);
-	annotation_add_state(&aub_annotations, AUB_TRACE_BINDING_TABLE,
-			     offset, 8);
+	annotation_add_state(aub, AUB_TRACE_BINDING_TABLE, offset, 8);
 
 	binding_table[0] =
-		gen8_bind_buf(batch, dst, GEN6_SURFACEFORMAT_B8G8R8A8_UNORM, 1);
+		gen8_bind_buf(batch, aub,
+			      dst, GEN6_SURFACEFORMAT_B8G8R8A8_UNORM, 1);
 	binding_table[1] =
-		gen8_bind_buf(batch, src, GEN6_SURFACEFORMAT_B8G8R8A8_UNORM, 0);
+		gen8_bind_buf(batch, aub,
+			      src, GEN6_SURFACEFORMAT_B8G8R8A8_UNORM, 0);
 
 	return offset;
 }
 
 /* Mostly copy+paste from gen6, except wrap modes moved */
 static uint32_t
-gen8_create_sampler(struct intel_batchbuffer *batch) {
+gen8_create_sampler(struct intel_batchbuffer *batch,
+		    struct annotations_context *aub)
+{
 	struct gen8_sampler_state *ss;
 	uint32_t offset;
 
 	ss = batch_alloc(batch, sizeof(*ss), 64);
 	offset = batch_offset(batch, ss);
-	annotation_add_state(&aub_annotations, AUB_TRACE_SAMPLER_STATE,
+	annotation_add_state(aub, AUB_TRACE_SAMPLER_STATE,
 			     offset, sizeof(*ss));
 
 	ss->ss0.min_filter = GEN6_MAPFILTER_NEAREST;
@@ -255,19 +279,19 @@ gen8_create_sampler(struct intel_batchbuffer *batch) {
 
 static uint32_t
 gen8_fill_ps(struct intel_batchbuffer *batch,
+	     struct annotations_context *aub,
 	     const uint32_t kernel[][4],
 	     size_t size)
 {
 	uint32_t offset;
 
 	offset = batch_copy(batch, kernel, size, 64);
-	annotation_add_state(&aub_annotations, AUB_TRACE_KERNEL_INSTRUCTIONS,
-			     offset, size);
+	annotation_add_state(aub, AUB_TRACE_KERNEL_INSTRUCTIONS, offset, size);
 
 	return offset;
 }
 
-/**
+/*
  * gen7_fill_vertex_buffer_data populate vertex buffer with data.
  *
  * The vertex buffer consists of 3 vertices to construct a RECTLIST. The 4th
@@ -279,7 +303,8 @@ gen8_fill_ps(struct intel_batchbuffer *batch,
  */
 static uint32_t
 gen7_fill_vertex_buffer_data(struct intel_batchbuffer *batch,
-			     struct scratch_buf *src,
+			     struct annotations_context *aub,
+			     struct igt_buf *src,
 			     uint32_t src_x, uint32_t src_y,
 			     uint32_t dst_x, uint32_t dst_y,
 			     uint32_t width, uint32_t height)
@@ -291,24 +316,24 @@ gen7_fill_vertex_buffer_data(struct intel_batchbuffer *batch,
 	start = batch->ptr;
 
 	emit_vertex_2s(batch, dst_x + width, dst_y + height);
-	emit_vertex_normalized(batch, src_x + width, buf_width(src));
-	emit_vertex_normalized(batch, src_y + height, buf_height(src));
+	emit_vertex_normalized(batch, src_x + width, igt_buf_width(src));
+	emit_vertex_normalized(batch, src_y + height, igt_buf_height(src));
 
 	emit_vertex_2s(batch, dst_x, dst_y + height);
-	emit_vertex_normalized(batch, src_x, buf_width(src));
-	emit_vertex_normalized(batch, src_y + height, buf_height(src));
+	emit_vertex_normalized(batch, src_x, igt_buf_width(src));
+	emit_vertex_normalized(batch, src_y + height, igt_buf_height(src));
 
 	emit_vertex_2s(batch, dst_x, dst_y);
-	emit_vertex_normalized(batch, src_x, buf_width(src));
-	emit_vertex_normalized(batch, src_y, buf_height(src));
+	emit_vertex_normalized(batch, src_x, igt_buf_width(src));
+	emit_vertex_normalized(batch, src_y, igt_buf_height(src));
 
 	offset = batch_offset(batch, start);
-	annotation_add_state(&aub_annotations, AUB_TRACE_VERTEX_BUFFER,
+	annotation_add_state(aub, AUB_TRACE_VERTEX_BUFFER,
 			     offset, 3 * VERTEX_SIZE);
 	return offset;
 }
 
-/**
+/*
  * gen6_emit_vertex_elements - The vertex elements describe the contents of the
  * vertex buffer. We pack the vertex buffer in a semi weird way, conforming to
  * what gen6_rendercopy did. The most straightforward would be to store
@@ -365,39 +390,40 @@ gen6_emit_vertex_elements(struct intel_batchbuffer *batch) {
 		  GEN6_VFCOMPONENT_STORE_1_FLT << VE1_VFCOMPONENT_3_SHIFT);
 }
 
-/**
- * gen7_emit_vertex_buffer emit the vertex buffers command
+/*
+ * gen8_emit_vertex_buffer emit the vertex buffers command
  *
  * @batch
  * @offset - bytw offset within the @batch where the vertex buffer starts.
  */
-static void gen7_emit_vertex_buffer(struct intel_batchbuffer *batch,
+static void gen8_emit_vertex_buffer(struct intel_batchbuffer *batch,
 				    uint32_t offset) {
 	OUT_BATCH(GEN6_3DSTATE_VERTEX_BUFFERS | (1 + (4 * 1) - 2));
 	OUT_BATCH(0 << VB0_BUFFER_INDEX_SHIFT | /* VB 0th index */
 		  GEN7_VB0_BUFFER_ADDR_MOD_EN | /* Address Modify Enable */
 		  VERTEX_SIZE << VB0_BUFFER_PITCH_SHIFT);
 	OUT_RELOC(batch->bo, I915_GEM_DOMAIN_VERTEX, 0, offset);
-	OUT_BATCH(0);
 	OUT_BATCH(3 * VERTEX_SIZE);
 }
 
 static uint32_t
-gen6_create_cc_state(struct intel_batchbuffer *batch)
+gen6_create_cc_state(struct intel_batchbuffer *batch,
+		     struct annotations_context *aub)
 {
 	struct gen6_color_calc_state *cc_state;
 	uint32_t offset;
 
 	cc_state = batch_alloc(batch, sizeof(*cc_state), 64);
 	offset = batch_offset(batch, cc_state);
-	annotation_add_state(&aub_annotations, AUB_TRACE_CC_STATE,
+	annotation_add_state(aub, AUB_TRACE_CC_STATE,
 			     offset, sizeof(*cc_state));
 
 	return offset;
 }
 
 static uint32_t
-gen8_create_blend_state(struct intel_batchbuffer *batch)
+gen8_create_blend_state(struct intel_batchbuffer *batch,
+			struct annotations_context *aub)
 {
 	struct gen8_blend_state *blend;
 	int i;
@@ -405,7 +431,7 @@ gen8_create_blend_state(struct intel_batchbuffer *batch)
 
 	blend = batch_alloc(batch, sizeof(*blend), 64);
 	offset = batch_offset(batch, blend);
-	annotation_add_state(&aub_annotations, AUB_TRACE_BLEND_STATE,
+	annotation_add_state(aub, AUB_TRACE_BLEND_STATE,
 			     offset, sizeof(*blend));
 
 	for (i = 0; i < 16; i++) {
@@ -420,14 +446,15 @@ gen8_create_blend_state(struct intel_batchbuffer *batch)
 }
 
 static uint32_t
-gen6_create_cc_viewport(struct intel_batchbuffer *batch)
+gen6_create_cc_viewport(struct intel_batchbuffer *batch,
+			struct annotations_context *aub)
 {
 	struct gen6_cc_viewport *vp;
 	uint32_t offset;
 
 	vp = batch_alloc(batch, sizeof(*vp), 32);
 	offset = batch_offset(batch, vp);
-	annotation_add_state(&aub_annotations, AUB_TRACE_CC_VP_STATE,
+	annotation_add_state(aub, AUB_TRACE_CC_VP_STATE,
 			     offset, sizeof(*vp));
 
 	/* XXX I don't understand this */
@@ -438,14 +465,16 @@ gen6_create_cc_viewport(struct intel_batchbuffer *batch)
 }
 
 static uint32_t
-gen7_create_sf_clip_viewport(struct intel_batchbuffer *batch) {
+gen7_create_sf_clip_viewport(struct intel_batchbuffer *batch,
+			struct annotations_context *aub)
+{
 	/* XXX these are likely not needed */
 	struct gen7_sf_clip_viewport *scv_state;
 	uint32_t offset;
 
 	scv_state = batch_alloc(batch, sizeof(*scv_state), 64);
 	offset = batch_offset(batch, scv_state);
-	annotation_add_state(&aub_annotations, AUB_TRACE_CLIP_VP_STATE,
+	annotation_add_state(aub, AUB_TRACE_CLIP_VP_STATE,
 			     offset, sizeof(*scv_state));
 
 	scv_state->guardband.xmin = 0;
@@ -457,14 +486,15 @@ gen7_create_sf_clip_viewport(struct intel_batchbuffer *batch) {
 }
 
 static uint32_t
-gen6_create_scissor_rect(struct intel_batchbuffer *batch)
+gen6_create_scissor_rect(struct intel_batchbuffer *batch,
+			struct annotations_context *aub)
 {
 	struct gen6_scissor_rect *scissor;
 	uint32_t offset;
 
 	scissor = batch_alloc(batch, sizeof(*scissor), 64);
 	offset = batch_offset(batch, scissor);
-	annotation_add_state(&aub_annotations, AUB_TRACE_SCISSOR_STATE,
+	annotation_add_state(aub, AUB_TRACE_SCISSOR_STATE,
 			     offset, sizeof(*scissor));
 
 	return offset;
@@ -504,12 +534,10 @@ gen8_emit_state_base_address(struct intel_batchbuffer *batch) {
 
 	/* surface */
 	OUT_RELOC(batch->bo, I915_GEM_DOMAIN_SAMPLER, 0, BASE_ADDRESS_MODIFY);
-	OUT_BATCH(0);
 
 	/* dynamic */
 	OUT_RELOC(batch->bo, I915_GEM_DOMAIN_RENDER | I915_GEM_DOMAIN_INSTRUCTION,
 		  0, BASE_ADDRESS_MODIFY);
-	OUT_BATCH(0);
 
 	/* indirect */
 	OUT_BATCH(0);
@@ -517,7 +545,6 @@ gen8_emit_state_base_address(struct intel_batchbuffer *batch) {
 
 	/* instruction */
 	OUT_RELOC(batch->bo, I915_GEM_DOMAIN_INSTRUCTION, 0, BASE_ADDRESS_MODIFY);
-	OUT_BATCH(0);
 
 	/* general state buffer size */
 	OUT_BATCH(0xfffff000 | 1);
@@ -776,7 +803,7 @@ gen8_emit_ps(struct intel_batchbuffer *batch, uint32_t kernel) {
 	OUT_BATCH(GEN7_3DSTATE_PS | (12-2));
 	OUT_BATCH(kernel);
 	OUT_BATCH(0); /* kernel hi */
-	OUT_BATCH(1 << GEN6_3DSTATE_WM_SAMPLER_COUNT_SHITF |
+	OUT_BATCH(1 << GEN6_3DSTATE_WM_SAMPLER_COUNT_SHIFT |
 		  2 << GEN6_3DSTATE_WM_BINDING_TABLE_ENTRY_COUNT_SHIFT);
 	OUT_BATCH(0); /* scratch space stuff */
 	OUT_BATCH(0); /* scratch hi */
@@ -797,6 +824,10 @@ gen8_emit_ps(struct intel_batchbuffer *batch, uint32_t kernel) {
 
 static void
 gen8_emit_depth(struct intel_batchbuffer *batch) {
+	OUT_BATCH(GEN8_3DSTATE_WM_DEPTH_STENCIL | (3 - 2));
+	OUT_BATCH(0);
+	OUT_BATCH(0);
+
 	OUT_BATCH(GEN7_3DSTATE_DEPTH_BUFFER | (8-2));
 	OUT_BATCH(0);
 	OUT_BATCH(0);
@@ -827,11 +858,11 @@ gen7_emit_clear(struct intel_batchbuffer *batch) {
 }
 
 static void
-gen6_emit_drawing_rectangle(struct intel_batchbuffer *batch, struct scratch_buf *dst)
+gen6_emit_drawing_rectangle(struct intel_batchbuffer *batch, struct igt_buf *dst)
 {
 	OUT_BATCH(GEN6_3DSTATE_DRAWING_RECTANGLE | (4 - 2));
 	OUT_BATCH(0);
-	OUT_BATCH((buf_height(dst) - 1) << 16 | (buf_width(dst) - 1));
+	OUT_BATCH((igt_buf_height(dst) - 1) << 16 | (igt_buf_width(dst) - 1));
 	OUT_BATCH(0);
 }
 
@@ -891,10 +922,11 @@ static void gen8_emit_primitive(struct intel_batchbuffer *batch, uint32_t offset
 
 void gen8_render_copyfunc(struct intel_batchbuffer *batch,
 			  drm_intel_context *context,
-			  struct scratch_buf *src, unsigned src_x, unsigned src_y,
+			  struct igt_buf *src, unsigned src_x, unsigned src_y,
 			  unsigned width, unsigned height,
-			  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y)
+			  struct igt_buf *dst, unsigned dst_x, unsigned dst_y)
 {
+	struct annotations_context aub_annotations;
 	uint32_t ps_sampler_state, ps_kernel_off, ps_binding_table;
 	uint32_t scissor_state;
 	uint32_t vertex_buffer;
@@ -908,21 +940,24 @@ void gen8_render_copyfunc(struct intel_batchbuffer *batch,
 
 	annotation_init(&aub_annotations);
 
-	ps_binding_table  = gen8_bind_surfaces(batch, src, dst);
-	ps_sampler_state  = gen8_create_sampler(batch);
-	ps_kernel_off = gen8_fill_ps(batch, ps_kernel, sizeof(ps_kernel));
-	vertex_buffer = gen7_fill_vertex_buffer_data(batch, src,
+	ps_binding_table  = gen8_bind_surfaces(batch, &aub_annotations,
+					       src, dst);
+	ps_sampler_state  = gen8_create_sampler(batch, &aub_annotations);
+	ps_kernel_off = gen8_fill_ps(batch, &aub_annotations,
+				     ps_kernel, sizeof(ps_kernel));
+	vertex_buffer = gen7_fill_vertex_buffer_data(batch, &aub_annotations,
+						     src,
 						     src_x, src_y,
 						     dst_x, dst_y,
 						     width, height);
-	cc.cc_state = gen6_create_cc_state(batch);
-	cc.blend_state = gen8_create_blend_state(batch);
-	viewport.cc_state = gen6_create_cc_viewport(batch);
-	viewport.sf_clip_state = gen7_create_sf_clip_viewport(batch);
-	scissor_state = gen6_create_scissor_rect(batch);
+	cc.cc_state = gen6_create_cc_state(batch, &aub_annotations);
+	cc.blend_state = gen8_create_blend_state(batch, &aub_annotations);
+	viewport.cc_state = gen6_create_cc_viewport(batch, &aub_annotations);
+	viewport.sf_clip_state = gen7_create_sf_clip_viewport(batch, &aub_annotations);
+	scissor_state = gen6_create_scissor_rect(batch, &aub_annotations);
 	/* TODO: theree is other state which isn't setup */
 
-	assert(batch->ptr < &batch->buffer[4095]);
+	igt_assert(batch->ptr < &batch->buffer[4095]);
 
 	batch->ptr = batch->buffer;
 
@@ -976,7 +1011,7 @@ void gen8_render_copyfunc(struct intel_batchbuffer *batch,
 
 	gen6_emit_drawing_rectangle(batch, dst);
 
-	gen7_emit_vertex_buffer(batch, vertex_buffer);
+	gen8_emit_vertex_buffer(batch, vertex_buffer);
 	gen6_emit_vertex_elements(batch);
 
 	gen8_emit_vf_topology(batch);
@@ -985,7 +1020,7 @@ void gen8_render_copyfunc(struct intel_batchbuffer *batch,
 	OUT_BATCH(MI_BATCH_BUFFER_END);
 
 	batch_end = batch_align(batch, 8);
-	assert(batch_end < BATCH_STATE_SPLIT);
+	igt_assert(batch_end < BATCH_STATE_SPLIT);
 	annotation_add_batch(&aub_annotations, batch_end);
 
 	dump_batch(batch);
