@@ -54,6 +54,8 @@
 #include "intel_chipset.h"
 #include "igt_aux.h"
 
+IGT_TEST_DESCRIPTION("Test the CS prefetch behaviour on batches.");
+
 static drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
 
@@ -88,7 +90,7 @@ static void exec(int fd, uint32_t handle)
 		       DRM_IOCTL_I915_GEM_EXECBUFFER2,
 		       &execbuf);
 	gem_sync(fd, handle);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 }
 
 igt_simple_main
