@@ -47,6 +47,8 @@
 #include "intel_io.h"
 #include "intel_chipset.h"
 
+IGT_TEST_DESCRIPTION("Basic test for memory and refcount leaks.");
+
 /* options */
 int num_contexts = 10;
 int uncontexted = 0; /* test only context create/destroy */
@@ -83,7 +85,7 @@ static void *work(void *arg)
 	else
 		td_fd = fd;
 
-	igt_assert(td_fd >= 0);
+	igt_assert_lte(0, td_fd);
 
 	bufmgr = drm_intel_bufmgr_gem_init(td_fd, 4096);
 	batch = intel_batchbuffer_alloc(bufmgr, devid);
@@ -103,7 +105,7 @@ static void *work(void *arg)
 		} else {
 			int ret;
 			ret = drm_intel_bo_subdata(batch->bo, 0, 4096, batch->buffer);
-			igt_assert(ret == 0);
+			igt_assert_eq(ret, 0);
 			intel_batchbuffer_flush_with_context(batch, context);
 		}
 	}
@@ -142,7 +144,7 @@ int main(int argc, char *argv[])
 {
 	int i;
 
-	igt_simple_init_parse_opts(argc, argv, "i:c:n:mu", NULL, NULL,
+	igt_simple_init_parse_opts(&argc, argv, "i:c:n:mu", NULL, NULL,
 				   opt_handler);
 
 	fd = drm_open_any_render();
@@ -166,5 +168,5 @@ int main(int argc, char *argv[])
 	free(threads);
 	close(fd);
 
-	return 0;
+	igt_exit();
 }
