@@ -27,37 +27,45 @@
 
 #include <stdio.h>
 #include <string.h>
-#include "i915_drm.h"
+#include <errno.h>
+
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 
-struct local_drm_i915_gem_context_create {
-	__u32 ctx_id;
-	__u32 pad;
-};
+int ret, fd;
+struct drm_i915_gem_context_create create;
 
-#define CONTEXT_CREATE_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2d, struct local_drm_i915_gem_context_create)
-
-igt_simple_main
+igt_main
 {
-	int ret, fd;
-	struct local_drm_i915_gem_context_create create;
+	igt_fixture
+		fd = drm_open_any_render();
 
-	igt_skip_on_simulation();
+	igt_subtest("basic") {
+		create.ctx_id = rand();
+		create.pad = 0;
 
-	create.ctx_id = rand();
-	create.pad = rand();
 
-	fd = drm_open_any_render();
-
-	ret = drmIoctl(fd, CONTEXT_CREATE_IOCTL, &create);
-	if (ret != 0 && (errno == ENODEV || errno == EINVAL)) {
-		igt_skip("Kernel is too old, or contexts not supported: %s\n",
-			 strerror(errno));
-	} else if (ret != 0) {
-		fprintf(stderr, "%s\n", strerror(errno));
-		igt_fail(1);
+		ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create);
+		igt_skip_on(ret != 0 && (errno == ENODEV || errno == EINVAL));
+		igt_assert(ret == 0);
+		igt_assert(create.ctx_id != 0);
 	}
-	igt_assert(create.ctx_id != 0);
 
-	close(fd);
+	igt_subtest("invalid-pad") {
+		create.ctx_id = rand();
+		create.pad = 0;
+
+		ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create);
+		igt_skip_on(ret != 0 && (errno == ENODEV || errno == EINVAL));
+		igt_assert(ret == 0);
+		igt_assert(create.ctx_id != 0);
+
+		create.pad = 1;
+
+		igt_assert(drmIoctl(fd, DRM_IOCTL_I915_GEM_CONTEXT_CREATE, &create) < 0 &&
+			   errno == EINVAL);
+	}
+
+	igt_fixture
+		close(fd);
 }
