@@ -34,17 +34,21 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_bufmgr.h"
 #include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
+#include "intel_chipset.h"
 
 /* Testcase: Test whether the kernel rejects relocations with non-gpu domains
  *
  * If it does not, it'll oops somewhen later on because we don't expect that.
  */
 
+IGT_TEST_DESCRIPTION("Test whether the kernel rejects relocations with non-gpu"
+		     " domains.");
+
 static drm_intel_bufmgr *bufmgr;
 struct intel_batchbuffer *batch;
 
@@ -71,7 +75,7 @@ run_batch(void)
 	used = batch->ptr - batch->buffer;
 
 	ret = drm_intel_bo_subdata(batch->bo, 0, used, batch->buffer);
-	igt_assert(ret == 0);
+	igt_assert_eq(ret, 0);
 
 	batch->ptr = NULL;
 
@@ -140,11 +144,10 @@ static void multi_write_domain(int fd)
 	ret = drmIoctl(fd,
 		       DRM_IOCTL_I915_GEM_EXECBUFFER2,
 		       &execbuf);
+	igt_assert(ret != 0 && errno == EINVAL);
 
 	gem_close(fd, handle);
 	gem_close(fd, handle_target);
-
-	igt_assert(ret != 0 && errno == EINVAL);
 }
 
 int fd;
@@ -163,13 +166,13 @@ igt_main
 	}
 
 	igt_subtest("cpu-domain") {
-		BEGIN_BATCH(2);
+		BEGIN_BATCH(2, 1);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, I915_GEM_DOMAIN_CPU, 0, 0);
 		ADVANCE_BATCH();
 		igt_assert(run_batch() == -EINVAL);
 
-		BEGIN_BATCH(2);
+		BEGIN_BATCH(2, 1);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU, 0);
 		ADVANCE_BATCH();
@@ -177,13 +180,13 @@ igt_main
 	}
 
 	igt_subtest("gtt-domain") {
-		BEGIN_BATCH(2);
+		BEGIN_BATCH(2, 1);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, I915_GEM_DOMAIN_GTT, 0, 0);
 		ADVANCE_BATCH();
 		igt_assert(run_batch() == -EINVAL);
 
-		BEGIN_BATCH(2);
+		BEGIN_BATCH(2, 1);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT, 0);
 		ADVANCE_BATCH();
@@ -193,7 +196,7 @@ igt_main
 	/* Note: Older kernels disallow this. Punt on the skip check though
 	 * since this is too old. */
 	igt_subtest("conflicting-write-domain") {
-		BEGIN_BATCH(4);
+		BEGIN_BATCH(4, 2);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, I915_GEM_DOMAIN_RENDER,
 			  I915_GEM_DOMAIN_RENDER, 0);
@@ -208,14 +211,14 @@ igt_main
 		multi_write_domain(fd);
 
 	igt_subtest("invalid-gpu-domain") {
-		BEGIN_BATCH(2);
+		BEGIN_BATCH(2, 1);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, ~(I915_GEM_GPU_DOMAINS | I915_GEM_DOMAIN_GTT | I915_GEM_DOMAIN_CPU),
 			  0, 0);
 		ADVANCE_BATCH();
 		igt_assert(run_batch() == -EINVAL);
 
-		BEGIN_BATCH(2);
+		BEGIN_BATCH(2, 1);
 		OUT_BATCH(0);
 		OUT_RELOC(tmp, I915_GEM_DOMAIN_GTT << 1,
 			  I915_GEM_DOMAIN_GTT << 1, 0);
