@@ -37,10 +37,10 @@
 #include <sys/ioctl.h>
 #include <sys/time.h>
 #include "drm.h"
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
 #include "intel_chipset.h"
-#include "intel_gpu_tools.h"
+#include "intel_io.h"
 
 #define OBJECT_SIZE 16384
 
@@ -60,7 +60,7 @@ static int gem_linear_blt(int fd,
 	uint32_t *b = batch;
 	int height = length / (16 * 1024);
 
-	igt_assert(height <= 1<<16);
+	igt_assert_lte(height, 1 << 16);
 
 	if (height) {
 		int i = 0;
@@ -170,7 +170,23 @@ static const char *bytes_per_sec(char *buf, double v)
 	return buf;
 }
 
-static void run(int object_size)
+static uint32_t dumb_create(int fd)
+{
+	struct drm_mode_create_dumb arg;
+	int ret;
+
+	arg.bpp = 32;
+	arg.width = 32;
+	arg.height = 32;
+
+	ret = drmIoctl(fd, DRM_IOCTL_MODE_CREATE_DUMB, &arg);
+	igt_assert_eq(ret, 0);
+	igt_assert(arg.size >= 4096);
+
+	return arg.handle;
+}
+
+static void run(int object_size, bool dumb)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec[3];
@@ -181,7 +197,11 @@ static void run(int object_size)
 	int ring;
 
 	fd = drm_open_any();
-	handle = gem_create(fd, 4096);
+	if (dumb)
+		handle = dumb_create(fd);
+	else
+		handle = gem_create(fd, 4096);
+
 	src = gem_create(fd, object_size);
 	dst = gem_create(fd, object_size);
 
@@ -242,10 +262,10 @@ static void run(int object_size)
 			gem_execbuf(fd, &execbuf);
 		gem_sync(fd, handle);
 		gettimeofday(&end, NULL);
-		printf("Time to blt %d bytes x %6d:	%7.3fµs, %s\n",
-		       object_size, count,
-		       elapsed(&start, &end, count),
-		       bytes_per_sec((char *)buf, object_size/elapsed(&start, &end, count)*1e6));
+		igt_info("Time to blt %d bytes x %6d:	%7.3fµs, %s\n",
+			 object_size, count,
+			 elapsed(&start, &end, count),
+			 bytes_per_sec((char *)buf, object_size/elapsed(&start, &end, count)*1e6));
 		fflush(stdout);
 	}
 	gem_close(fd, handle);
@@ -257,18 +277,31 @@ int main(int argc, char **argv)
 {
 	int i;
 
-	igt_simple_init();
+	igt_subtest_init(argc, argv);
 
 	igt_skip_on_simulation();
 
-	if (argc > 1) {
-		for (i = 1; i < argc; i++) {
-			int object_size = atoi(argv[i]);
-			if (object_size)
-				run((object_size + 3) & -4);
-		}
-	} else
-		run(OBJECT_SIZE);
+	igt_subtest("normal") {
+		if (argc > 1) {
+			for (i = 1; i < argc; i++) {
+				int object_size = atoi(argv[i]);
+				if (object_size)
+					run((object_size + 3) & -4, false);
+			}
+		} else
+			run(OBJECT_SIZE, false);
+	}
 
-	return 0;
+	igt_subtest("dumb-buf") {
+		if (argc > 1) {
+			for (i = 1; i < argc; i++) {
+				int object_size = atoi(argv[i]);
+				if (object_size)
+					run((object_size + 3) & -4, true);
+			}
+		} else
+			run(OBJECT_SIZE, true);
+	}
+
+	igt_exit();
 }
