@@ -46,6 +46,8 @@
 #include "igt_debugfs.h"
 #include "igt_aux.h"
 
+IGT_TEST_DESCRIPTION("Test persistent relocations as used by uxa/libva.");
+
 /*
  * Testcase: Persistent relocations as used by uxa/libva
  *
@@ -103,7 +105,7 @@ static drm_intel_bo *create_special_bo(void)
 		BATCH(CMD_POLY_STIPPLE_OFFSET << 16);
 		BATCH(0);
 	}
-	igt_assert(len % 2 == 0);
+	igt_assert_eq(len % 2, 0);
 	BATCH(MI_NOOP);
 	BATCH(MI_BATCH_BUFFER_END);
 
