@@ -58,7 +58,6 @@
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#include <getopt.h>
 
 #include <drm.h>
 
@@ -70,8 +69,14 @@
 #include "intel_chipset.h"
 #include "igt_aux.h"
 
+IGT_TEST_DESCRIPTION("General gem coherency test.");
+
 #define CMD_POLY_STIPPLE_OFFSET       0x7906
 
+#define DUCTAPE 0xdead0001
+#define TILESZ	0xdead0002
+#define CHCK_RENDER 0xdead0003
+
 /** TODO:
  * - beat on relaxed fencing (i.e. mappable/fenceable tracking in the kernel)
  * - render copy (to check fence tracking and cache coherency management by the
@@ -160,22 +165,20 @@ static void emit_blt(drm_intel_bo *src_bo, uint32_t src_tiling, unsigned src_pit
 	}
 
 	/* copy lower half to upper half */
-	BLIT_COPY_BATCH_START(devid, cmd_bits);
+	BLIT_COPY_BATCH_START(cmd_bits);
 	OUT_BATCH((3 << 24) | /* 32 bits */
 		  (0xcc << 16) | /* copy ROP */
 		  dst_pitch);
 	OUT_BATCH(dst_y << 16 | dst_x);
 	OUT_BATCH((dst_y+h) << 16 | (dst_x+w));
 	OUT_RELOC_FENCED(dst_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-	BLIT_RELOC_UDW(devid);
 	OUT_BATCH(src_y << 16 | src_x);
 	OUT_BATCH(src_pitch);
 	OUT_RELOC_FENCED(src_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
-	BLIT_RELOC_UDW(devid);
 	ADVANCE_BATCH();
 
-	if (IS_GEN6(devid) || IS_GEN7(devid)) {
-		BEGIN_BATCH(3);
+	if (batch->gen >= 6) {
+		BEGIN_BATCH(3, 0);
 		OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
 		OUT_BATCH(0);
 		OUT_BATCH(0);
@@ -190,7 +193,7 @@ static void keep_gpu_busy(void)
 	int tmp;
 
 	tmp = 1 << gpu_busy_load;
-	igt_assert(tmp <= 1024);
+	igt_assert_lte(tmp, 1024);
 
 	emit_blt(busy_bo, 0, 4096, 0, 0, tmp, 128,
 		 busy_bo, 0, 4096, 0, 128);
@@ -223,18 +226,15 @@ static void cpucpy2d(uint32_t *src, unsigned src_stride, unsigned src_x, unsigne
 			    + i*options.tile_size + j;
 			uint32_t tmp = src[src_ofs];
 			if (tmp != expect) {
-			    printf("mismatch at tile %i pos %i, read %i, expected %i, diff %i\n",
-				    logical_tile_no, i*options.tile_size + j, tmp, expect, (int) tmp - expect);
-			    if (options.trace_tile >= 0 && options.fail)
-				    igt_fail(1);
+			    igt_info("mismatch at tile %i pos %i, read %i, expected %i, diff %i\n", logical_tile_no, i * options.tile_size + j, tmp, expect, (int)tmp - expect);
+			    igt_fail_on(options.trace_tile >= 0 && options.fail);
 			    failed++;
 			}
 			/* when not aborting, correct any errors */
 			dst[dst_ofs] = expect;
 		}
 	}
-	if (failed && options.fail)
-		igt_fail(1);
+	igt_fail_on(failed && options.fail);
 
 	if (failed > stats.max_failed_reads)
 		stats.max_failed_reads = failed;
@@ -370,35 +370,35 @@ static void next_copyfunc(int tile)
 {
 	if (fence_storm) {
 		if (tile == options.trace_tile)
-			printf(" using fence storm\n");
+			igt_info(" using fence storm\n");
 		return;
 	}
 
 	if (copyfunc_seq % 61 == 0
 			&& options.forced_tiling != I915_TILING_NONE) {
 		if (tile == options.trace_tile)
-			printf(" using fence storm\n");
+			igt_info(" using fence storm\n");
 		fence_storm = num_fences;
 		copyfunc = blitter_copyfunc;
 	} else if (copyfunc_seq % 17 == 0) {
 		if (tile == options.trace_tile)
-			printf(" using cpu\n");
+			igt_info(" using cpu\n");
 		copyfunc = cpu_copyfunc;
 	} else if (copyfunc_seq % 19 == 0) {
 		if (tile == options.trace_tile)
-			printf(" using prw\n");
+			igt_info(" using prw\n");
 		copyfunc = prw_copyfunc;
 	} else if (copyfunc_seq % 3 == 0 && options.use_render) {
 		if (tile == options.trace_tile)
-			printf(" using render\n");
+			igt_info(" using render\n");
 		copyfunc = render_copyfunc;
 	} else if (options.use_blt){
 		if (tile == options.trace_tile)
-			printf(" using blitter\n");
+			igt_info(" using blitter\n");
 		copyfunc = blitter_copyfunc;
 	} else if (options.use_render){
 		if (tile == options.trace_tile)
-			printf(" using render\n");
+			igt_info(" using render\n");
 		copyfunc = render_copyfunc;
 	} else {
 		copyfunc = cpu_copyfunc;
@@ -563,20 +563,15 @@ static void init_set(unsigned set)
 			       buffers[set][i].stride);
 
 		if (options.trace_tile != -1 && i == options.trace_tile/options.tiles_per_buf)
-			printf("changing buffer %i containing tile %i: tiling %i, stride %i\n", i,
-					options.trace_tile,
-					buffers[set][i].tiling, buffers[set][i].stride);
+			igt_info("changing buffer %i containing tile %i: tiling %i, stride %i\n", i, options.trace_tile, buffers[set][i].tiling, buffers[set][i].stride);
 	}
 }
 
 static void exchange_uint(void *array, unsigned i, unsigned j)
 {
 	unsigned *i_arr = array;
-	unsigned i_tmp;
 
-	i_tmp = i_arr[i];
-	i_arr[i] = i_arr[j];
-	i_arr[j] = i_tmp;
+	igt_swap(i_arr[i], i_arr[j]);
 }
 
 static void copy_tiles(unsigned *permutation)
@@ -602,9 +597,7 @@ static void copy_tiles(unsigned *permutation)
 		tile2xy(dst_buf, dst_tile, &dst_x, &dst_y);
 
 		if (options.trace_tile == i)
-			printf("copying tile %i from %i (%i, %i) to %i (%i, %i)", i,
-				tile_permutation[i], src_buf_idx, src_tile,
-				permutation[idx], dst_buf_idx, dst_tile);
+			igt_info("copying tile %i from %i (%i, %i) to %i (%i, %i)", i, tile_permutation[i], src_buf_idx, src_tile, permutation[idx], dst_buf_idx, dst_tile);
 
 		if (options.no_hw) {
 			cpucpy2d(src_buf->data,
@@ -631,71 +624,27 @@ static void sanitize_tiles_per_buf(void)
 		options.tiles_per_buf = options.scratch_buf_size / TILE_BYTES(options.tile_size);
 }
 
-static void parse_options(int argc, char **argv)
+static int parse_options(int opt, int opt_index)
 {
-	int c, tmp;
-	int option_index = 0;
-	static struct option long_options[] = {
-		{"no-hw", 0, 0, 'd'},
-		{"buf-size", 1, 0, 's'},
-		{"gpu-busy-load", 1, 0, 'g'},
-		{"no-signals", 0, 0, 'S'},
-		{"buffer-count", 1, 0, 'c'},
-		{"trace-tile", 1, 0, 't'},
-		{"disable-blt", 0, 0, 'b'},
-		{"disable-render", 0, 0, 'r'},
-		{"untiled", 0, 0, 'u'},
-		{"x-tiled", 0, 0, 'x'},
-		{"use-cpu-maps", 0, 0, 'm'},
-		{"rounds", 1, 0, 'o'},
-		{"no-fail", 0, 0, 'f'},
-		{"tiles-per-buf", 0, 0, 'p'},
-#define DUCTAPE 0xdead0001
-		{"remove-duct-tape", 0, 0, DUCTAPE},
-#define TILESZ	0xdead0002
-		{"tile-size", 1, 0, TILESZ},
-#define CHCK_RENDER 0xdead0003
-		{"check-render-cpyfn", 0, 0, CHCK_RENDER},
-		{NULL, 0, 0, 0},
-	};
+	int tmp;
 
-	options.scratch_buf_size = 256*4096;
-	options.no_hw = 0;
-	options.use_signal_helper = 1;
-	options.gpu_busy_load = 0;
-	options.num_buffers = 0;
-	options.trace_tile = -1;
-	options.use_render = 1;
-	options.use_blt = 1;
-	options.forced_tiling = -1;
-	options.use_cpu_maps = 0;
-	options.total_rounds = 512;
-	options.fail = 1;
-	options.ducttape = 1;
-	options.tile_size = 16;
-	options.tiles_per_buf = options.scratch_buf_size / TILE_BYTES(options.tile_size);
-	options.check_render_cpyfn = 0;
-
-	while((c = getopt_long(argc, argv, "ds:g:c:t:rbuxmo:fp:",
-			       long_options, &option_index)) != -1) {
-		switch(c) {
+	switch(opt) {
 		case 'd':
 			options.no_hw = 1;
-			printf("no-hw debug mode\n");
+			igt_info("no-hw debug mode\n");
 			break;
 		case 'S':
 			options.use_signal_helper = 0;
-			printf("disabling that pesky nuisance who keeps interrupting us\n");
+			igt_info("disabling that pesky nuisance who keeps interrupting us\n");
 			break;
 		case 's':
 			tmp = atoi(optarg);
 			if (tmp < options.tile_size*8192)
-				printf("scratch buffer size needs to be at least %i\n",
-				       options.tile_size*8192);
+				igt_info("scratch buffer size needs to be at least %i\n", options.tile_size * 8192);
 			else if (tmp & (tmp - 1)) {
-				printf("scratch buffer size needs to be a power-of-two\n");
+				igt_info("scratch buffer size needs to be a power-of-two\n");
 			} else {
-				printf("fixed scratch buffer size to %u\n", tmp);
+				igt_info("fixed scratch buffer size to %u\n", tmp);
 				options.scratch_buf_size = tmp;
 				sanitize_tiles_per_buf();
 			}
@@ -703,79 +652,72 @@ static void parse_options(int argc, char **argv)
 		case 'g':
 			tmp = atoi(optarg);
 			if (tmp < 0 || tmp > 10)
-				printf("gpu busy load needs to be bigger than 0 and smaller than 10\n");
+				igt_info("gpu busy load needs to be bigger than 0 and smaller than 10\n");
 			else {
-				printf("gpu busy load factor set to %i\n", tmp);
+				igt_info("gpu busy load factor set to %i\n", tmp);
 				gpu_busy_load = options.gpu_busy_load = tmp;
 			}
 			break;
 		case 'c':
 			options.num_buffers = atoi(optarg);
-			printf("buffer count set to %i\n", options.num_buffers);
+			igt_info("buffer count set to %i\n", options.num_buffers);
 			break;
 		case 't':
 			options.trace_tile = atoi(optarg);
-			printf("tracing tile %i\n", options.trace_tile);
+			igt_info("tracing tile %i\n", options.trace_tile);
 			break;
 		case 'r':
 			options.use_render = 0;
-			printf("disabling render copy\n");
+			igt_info("disabling render copy\n");
 			break;
 		case 'b':
 			options.use_blt = 0;
-			printf("disabling blt copy\n");
+			igt_info("disabling blt copy\n");
 			break;
 		case 'u':
 			options.forced_tiling = I915_TILING_NONE;
-			printf("disabling tiling\n");
+			igt_info("disabling tiling\n");
 			break;
 		case 'x':
 			if (options.use_cpu_maps) {
-				printf("tiling not possible with cpu maps\n");
+				igt_info("tiling not possible with cpu maps\n");
 			} else {
 				options.forced_tiling = I915_TILING_X;
-				printf("using only X-tiling\n");
+				igt_info("using only X-tiling\n");
 			}
 			break;
 		case 'm':
 			options.use_cpu_maps = 1;
 			options.forced_tiling = I915_TILING_NONE;
-			printf("disabling tiling\n");
+			igt_info("disabling tiling\n");
 			break;
 		case 'o':
 			options.total_rounds = atoi(optarg);
-			printf("total rounds %i\n", options.total_rounds);
+			igt_info("total rounds %i\n", options.total_rounds);
 			break;
 		case 'f':
 			options.fail = 0;
-			printf("not failing when detecting errors\n");
+			igt_info("not failing when detecting errors\n");
 			break;
 		case 'p':
 			options.tiles_per_buf = atoi(optarg);
-			printf("tiles per buffer %i\n", options.tiles_per_buf);
+			igt_info("tiles per buffer %i\n", options.tiles_per_buf);
 			break;
 		case DUCTAPE:
 			options.ducttape = 0;
-			printf("applying duct-tape\n");
+			igt_info("applying duct-tape\n");
 			break;
 		case TILESZ:
 			options.tile_size = atoi(optarg);
 			sanitize_tiles_per_buf();
-			printf("til size %i\n", options.tile_size);
+			igt_info("til size %i\n", options.tile_size);
 			break;
 		case CHCK_RENDER:
 			options.check_render_cpyfn = 1;
-			printf("checking render copy function\n");
+			igt_info("checking render copy function\n");
 			break;
-		default:
-			printf("unkown command options\n");
-			break;
-		}
 	}
 
-	if (optind < argc)
-		printf("unkown command options\n");
-
 	/* actually 32767, according to docs, but that kills our nice pot calculations. */
 	options.max_dimension = 16*1024;
 	if (options.use_render) {
@@ -784,8 +726,9 @@ static void parse_options(int argc, char **argv)
 		else
 			options.max_dimension = 8192;
 	}
-	printf("Limiting buffer to %dx%d\n",
-	       options.max_dimension, options.max_dimension);
+	igt_info("Limiting buffer to %dx%d\n", options.max_dimension, options.max_dimension);
+
+	return 0;
 }
 
 static void init(void)
@@ -795,10 +738,10 @@ static void init(void)
 
 	if (options.num_buffers == 0) {
 		tmp = gem_aperture_size(drm_fd);
-		tmp = tmp > 256*(1024*1024) ? 256*(1024*1024) : tmp;
+		tmp = min(256 * (1024 * 1024), tmp);
 		num_buffers = 2 * tmp / options.scratch_buf_size / 3;
 		num_buffers /= 2;
-		printf("using %u buffers\n", num_buffers);
+		igt_info("using %u buffers\n", num_buffers);
 	} else
 		num_buffers = options.num_buffers;
 
@@ -806,7 +749,7 @@ static void init(void)
 	drm_intel_bufmgr_gem_enable_reuse(bufmgr);
 	drm_intel_bufmgr_gem_enable_fenced_relocs(bufmgr);
 	num_fences = gem_available_fences(drm_fd);
-	igt_assert(num_fences > 4);
+	igt_assert_lt(4, num_fences);
 	batch = intel_batchbuffer_alloc(bufmgr, devid);
 
 	busy_bo = drm_intel_bo_alloc(bufmgr, "tiled bo", BUSY_BUF_SIZE, 4096);
@@ -862,8 +805,7 @@ static void check_render_copyfunc(void)
 			ptr = (uint32_t*)((char *)dst.data + dx*4 + (dy+j) * dst.stride);
 			for (i = 0; i < options.tile_size; i++)
 				if (ptr[i] != j * options.tile_size + i) {
-					printf("render copyfunc mismatch at (%d, %d): found %d, expected %d\n",
-					       i, j, ptr[i], j*options.tile_size + i);
+					igt_info("render copyfunc mismatch at (%d, %d): found %d, expected %d\n", i, j, ptr[i], j * options.tile_size + i);
 				}
 		}
 	}
@@ -874,12 +816,50 @@ int main(int argc, char **argv)
 {
 	int i, j;
 	unsigned *current_permutation, *tmp_permutation;
+	static struct option long_options[] = {
+		{"no-hw", 0, 0, 'd'},
+		{"buf-size", 1, 0, 's'},
+		{"gpu-busy-load", 1, 0, 'g'},
+		{"no-signals", 0, 0, 'S'},
+		{"buffer-count", 1, 0, 'c'},
+		{"trace-tile", 1, 0, 't'},
+		{"disable-blt", 0, 0, 'b'},
+		{"disable-render", 0, 0, 'r'},
+		{"untiled", 0, 0, 'u'},
+		{"x-tiled", 0, 0, 'x'},
+		{"use-cpu-maps", 0, 0, 'm'},
+		{"rounds", 1, 0, 'o'},
+		{"no-fail", 0, 0, 'f'},
+		{"tiles-per-buf", 0, 0, 'p'},
+		{"remove-duct-tape", 0, 0, DUCTAPE},
+		{"tile-size", 1, 0, TILESZ},
+		{"check-render-cpyfn", 0, 0, CHCK_RENDER},
+		{NULL, 0, 0, 0},
+	};
+
+	options.scratch_buf_size = 256*4096;
+	options.no_hw = 0;
+	options.use_signal_helper = 1;
+	options.gpu_busy_load = 0;
+	options.num_buffers = 0;
+	options.trace_tile = -1;
+	options.use_render = 1;
+	options.use_blt = 1;
+	options.forced_tiling = -1;
+	options.use_cpu_maps = 0;
+	options.total_rounds = 512;
+	options.fail = 1;
+	options.ducttape = 1;
+	options.tile_size = 16;
+	options.tiles_per_buf = options.scratch_buf_size / TILE_BYTES(options.tile_size);
+	options.check_render_cpyfn = 0;
+
+	igt_simple_init_parse_opts(&argc, argv,"ds:g:c:t:rbuxmo:fp:",
+				   long_options, NULL, parse_options);
 
 	drm_fd = drm_open_any();
 	devid = intel_get_drm_devid(drm_fd);
 
-	parse_options(argc, argv);
-
 	/* start our little helper early before too may allocations occur */
 	if (options.use_signal_helper)
 		igt_fork_signal_helper();
@@ -898,10 +878,10 @@ int main(int argc, char **argv)
 	fan_out();
 
 	for (i = 0; i < options.total_rounds; i++) {
-		printf("round %i\n", i);
+		igt_info("round %i\n", i);
 		if (i % 64 == 63) {
 			fan_in_and_check();
-			printf("everything correct after %i rounds\n", i + 1);
+			igt_info("everything correct after %i rounds\n", i + 1);
 		}
 
 		target_set = (current_set + 1) & 1;
@@ -924,8 +904,7 @@ int main(int argc, char **argv)
 
 	fan_in_and_check();
 
-	fprintf(stderr, "num failed tiles %u, max incoherent bytes %zd\n",
-		stats.num_failed, stats.max_failed_reads*sizeof(uint32_t));
+	igt_info("num failed tiles %u, max incoherent bytes %zd\n", stats.num_failed, stats.max_failed_reads * sizeof(uint32_t));
 
 	intel_batchbuffer_free(batch);
 	drm_intel_bufmgr_destroy(bufmgr);
@@ -934,5 +913,5 @@ int main(int argc, char **argv)
 
 	igt_stop_signal_helper();
 
-	return 0;
+	igt_exit();
 }
