@@ -61,6 +61,9 @@
 #include "drmtest.h"
 #include "intel_io.h"
 #include "igt_aux.h"
+#include "igt_debugfs.h"
+
+IGT_TEST_DESCRIPTION("Exercise swizzle code for swapping.");
 
 #define WIDTH 512
 #define HEIGHT 512
@@ -68,29 +71,55 @@
 static uint32_t current_tiling_mode;
 
 #define PAGE_SIZE 4096
+#define AVAIL_RAM 512
 
 static uint32_t
-create_bo_and_fill(int fd)
+create_bo(int fd)
 {
 	uint32_t handle;
 	uint32_t *data;
-	int i;
 
 	handle = gem_create(fd, LINEAR_DWORDS);
 	gem_set_tiling(fd, handle, current_tiling_mode, WIDTH * sizeof(uint32_t));
 
-	/* Fill the BO with dwords starting at start_val */
 	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ | PROT_WRITE);
-	if (data == NULL && errno == ENOSPC)
+	if (data == NULL) {
+		gem_close(fd, handle);
 		return 0;
-
-	for (i = 0; i < WIDTH*HEIGHT; i++)
-		data[i] = i;
+	}
 	munmap(data, LINEAR_DWORDS);
 
 	return handle;
 }
 
+static void
+fill_bo(int fd, uint32_t handle)
+{
+	uint32_t *data;
+	int i;
+
+	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ | PROT_WRITE);
+	igt_assert(data);
+
+	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+	for (i = 0; i < WIDTH*HEIGHT; i++)
+		data[i] = i;
+	munmap(data, LINEAR_DWORDS);
+}
+
+static void
+check_bo(int fd, uint32_t handle)
+{
+	uint32_t *data;
+	int j;
+
+	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ);
+	gem_set_domain(fd, handle, I915_GEM_DOMAIN_GTT, 0);
+	j = rand() % (WIDTH * HEIGHT);
+	igt_assert_f(data[j] == j, "mismatch at %i: %i\n", j, data[j]);
+	munmap(data, LINEAR_DWORDS);
+}
+
 uint32_t *bo_handles;
 
 struct thread {
@@ -99,21 +128,6 @@ struct thread {
 	int fd, count;
 };
 
-static void
-check_bo(int fd, uint32_t handle)
-{
-	uint32_t *data;
-	int j;
-
-	/* Check the target bo's contents. */
-	data = gem_mmap(fd, handle, LINEAR_DWORDS, PROT_READ | PROT_WRITE);
-	for (j = 0; j < WIDTH*HEIGHT; j++)
-		igt_assert_f(data[j] == j,
-			     "mismatch at %i: %i\n",
-			     j, data[j]);
-	munmap(data, LINEAR_DWORDS);
-}
-
 static void *thread_run(void *data)
 {
 	struct thread *t = data;
@@ -145,54 +159,96 @@ static void thread_fini(struct thread *t)
 	free(t->idx_arr);
 }
 
-igt_simple_main
+static void check_memory_layout(void)
+{
+	FILE *tiling_debugfs_file;
+	char *line = NULL;
+	size_t sz = 0;
+
+	tiling_debugfs_file = igt_debugfs_fopen("i915_swizzle_info", "r");
+	igt_assert(tiling_debugfs_file);
+
+	while (getline(&line, &sz, tiling_debugfs_file) > 0) {
+		if (strstr(line, "L-shaped") != 0)
+			continue;
+
+		igt_skip("L-shaped memory configuration detected\n");
+	}
+
+	igt_debug("normal memory configuration detected, continuing\n");
+}
+
+igt_main
 {
 	struct thread *threads;
 	int fd, n, count, num_threads;
 
-	current_tiling_mode = I915_TILING_X;
+	igt_fixture {
+		size_t lock_size;
 
-	igt_skip_on_simulation();
-	intel_purge_vm_caches();
+		current_tiling_mode = I915_TILING_X;
 
-	fd = drm_open_any();
-	/* need slightly more than available memory */
-	count = intel_get_total_ram_mb() + intel_get_total_swap_mb() / 4;
-	bo_handles = calloc(count, sizeof(uint32_t));
-	igt_assert(bo_handles);
+		intel_purge_vm_caches();
 
-	num_threads = gem_available_fences(fd);
-	threads = calloc(num_threads, sizeof(struct thread));
-	igt_assert(threads);
+		fd = drm_open_any();
 
-	igt_log(IGT_LOG_INFO,
-		"Using %d 1MiB objects (available RAM: %ld/%ld, swap: %ld)\n",
-		count,
-		(long)intel_get_avail_ram_mb(),
-		(long)intel_get_total_ram_mb(),
-		(long)intel_get_total_swap_mb());
-	igt_require(intel_check_memory(count, 1024*1024, CHECK_RAM | CHECK_SWAP));
+		check_memory_layout();
 
-	for (n = 0; n < count; n++) {
-		bo_handles[n] = create_bo_and_fill(fd);
-		/* Not enough mmap address space possible. */
-		igt_require(bo_handles[n]);
+		/* lock RAM, leaving only 512MB available */
+		lock_size = max(0, intel_get_total_ram_mb() - AVAIL_RAM);
+		igt_lock_mem(lock_size);
+
+		/* need slightly more than available memory */
+		count = min(intel_get_total_ram_mb(), AVAIL_RAM) * 1.25;
+		bo_handles = calloc(count, sizeof(uint32_t));
+		igt_assert(bo_handles);
+
+		num_threads = gem_available_fences(fd);
+		threads = calloc(num_threads, sizeof(struct thread));
+		igt_assert(threads);
+
+		igt_info("Using %d 1MiB objects (available RAM: %ld/%ld, swap: %ld)\n",
+			 count,
+			 (long)intel_get_avail_ram_mb(),
+			 (long)intel_get_total_ram_mb(),
+			 (long)intel_get_total_swap_mb());
+		intel_require_memory(count, 1024*1024, CHECK_RAM | CHECK_SWAP);
+
+		for (n = 0; n < count; n++) {
+			bo_handles[n] = create_bo(fd);
+			/* Not enough mmap address space possible. */
+			igt_require(bo_handles[n]);
+		}
 	}
 
-	thread_init(&threads[0], fd, count);
-	thread_run(&threads[0]);
-	thread_fini(&threads[0]);
+	igt_subtest("non-threaded") {
+		for (n = 0; n < count; n++)
+			fill_bo(fd, bo_handles[n]);
+
+		thread_init(&threads[0], fd, count);
+		thread_run(&threads[0]);
+		thread_run(&threads[0]);
+		thread_run(&threads[0]);
+		thread_fini(&threads[0]);
+	}
 
 	/* Once more with threads */
 	igt_subtest("threaded") {
-		for (n = 0; n < num_threads; n++) {
+		for (n = 0; n < count; n++)
+			fill_bo(fd, bo_handles[n]);
+
+		for (n = 0; n < num_threads; n++)
 			thread_init(&threads[n], fd, count);
+
+		thread_run(&threads[0]);
+		for (n = 0; n < num_threads; n++)
 			pthread_create(&threads[n].thread, NULL, thread_run, &threads[n]);
-		}
-		for (n = 0; n < num_threads; n++) {
+		for (n = 0; n < num_threads; n++)
 			pthread_join(threads[n].thread, NULL);
+		thread_run(&threads[0]);
+
+		for (n = 0; n < num_threads; n++)
 			thread_fini(&threads[n]);
-		}
 	}
 
 	close(fd);
