@@ -34,13 +34,21 @@
 #include <sys/poll.h>
 #include <sys/time.h>
 #include <sys/ioctl.h>
+#ifdef HAVE_LINUX_KD_H
 #include <linux/kd.h>
+#elif HAVE_SYS_KD_H
+#include <sys/kd.h>
+#endif
+#include <time.h>
+#include <pthread.h>
 
-#include "i915_drm.h"
+#include "ioctl_wrappers.h"
 #include "drmtest.h"
-#include "testdisplay.h"
-#include "rendercopy.h"
+#include "intel_chipset.h"
+#include "intel_batchbuffer.h"
 #include "igt_kms.h"
+#include "igt_aux.h"
+#include "igt_gt.h"
 
 #define TEST_DPMS		(1 << 0)
 #define TEST_WITH_DUMMY_BCS	(1 << 1)
@@ -67,6 +75,11 @@
 #define TEST_ENOENT		(1 << 22)
 #define TEST_FENCE_STRESS	(1 << 23)
 #define TEST_VBLANK_RACE	(1 << 24)
+#define TEST_RPM		(1 << 25)
+#define TEST_SUSPEND		(1 << 26)
+#define TEST_TS_CONT		(1 << 27)
+#define TEST_BO_TOOBIG		(1 << 28)
+#define TEST_HANG_ONCE		(1 << 29)
 
 #define EVENT_FLIP		(1 << 0)
 #define EVENT_VBLANK		(1 << 1)
@@ -75,7 +88,8 @@
 #define DRM_CAP_TIMESTAMP_MONOTONIC 6
 #endif
 
-#define max(a, b) ((a) > (b) ? (a) : (b))
+#define USEC_PER_SEC 1000000L
+#define NSEC_PER_SEC 1000000000L
 
 drmModeRes *resources;
 int drm_fd;
@@ -135,7 +149,7 @@ struct test_output {
 	unsigned int fb_height;
 	unsigned int fb_ids[3];
 	int bpp, depth;
-	struct kmstest_fb fb_info[3];
+	struct igt_fb fb_info[3];
 
 	struct event_state flip_state;
 	struct event_state vblank_state;
@@ -155,57 +169,111 @@ static unsigned long gettime_us(void)
 	return ts.tv_sec * 1000000 + ts.tv_nsec / 1000;
 }
 
-static void emit_dummy_load__bcs(struct test_output *o)
+static int calibrate_dummy_load(struct test_output *o,
+				const char *ring_name,
+				int (*emit)(struct test_output *o, int limit, int timeout))
 {
-	int i, limit;
-	drm_intel_bo *dummy_bo, *target_bo, *tmp_bo;
-	struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
-	unsigned pitch = fb_info->stride;
+	unsigned long start;
+	int ops = 1;
 
-	limit = intel_gen(devid) < 6 ? 500 : 5000;
+	start = gettime_us();
 
-	dummy_bo = drm_intel_bo_alloc(bufmgr, "dummy_bo", fb_info->size, 4096);
-	igt_assert(dummy_bo);
-	target_bo = gem_handle_to_libdrm_bo(bufmgr, drm_fd, "imported", fb_info->gem_handle);
-	igt_assert(target_bo);
+	do {
+		unsigned long diff;
+		int ret;
+
+		ret = emit(o, (ops+1)/2, 10);
+		diff = gettime_us() - start;
+
+		if (ret || diff / USEC_PER_SEC >= 1)
+			break;
+
+		ops += ops;
+	} while (ops < 100000);
+
+	igt_debug("%s dummy load calibrated: %d operations / second\n",
+		  ring_name, ops);
+
+	return ops;
+}
+
+static void blit_copy(drm_intel_bo *dst, drm_intel_bo *src,
+		      unsigned int width, unsigned int height,
+		      unsigned int dst_pitch, unsigned int src_pitch)
+{
+	BLIT_COPY_BATCH_START(0);
+	OUT_BATCH((3 << 24) | /* 32 bits */
+		  (0xcc << 16) | /* copy ROP */
+		  dst_pitch);
+	OUT_BATCH(0 << 16 | 0);
+	OUT_BATCH(height << 16 | width);
+	OUT_RELOC_FENCED(dst, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
+	OUT_BATCH(0 << 16 | 0);
+	OUT_BATCH(src_pitch);
+	OUT_RELOC_FENCED(src, I915_GEM_DOMAIN_RENDER, 0, 0);
+	ADVANCE_BATCH();
+
+	if (batch->gen >= 6) {
+		BEGIN_BATCH(3, 0);
+		OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
+		OUT_BATCH(0);
+		OUT_BATCH(0);
+		ADVANCE_BATCH();
+	}
+}
+
+static int _emit_dummy_load__bcs(struct test_output *o, int limit, int timeout)
+{
+	int i, ret = 0;
+	drm_intel_bo *src_bo, *dst_bo, *fb_bo;
+	struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
+
+	src_bo = drm_intel_bo_alloc(bufmgr, "dummy_bo", 2048*2048*4, 4096);
+	igt_assert(src_bo);
+
+	dst_bo = drm_intel_bo_alloc(bufmgr, "dummy_bo", 2048*2048*4, 4096);
+	igt_assert(dst_bo);
+
+	fb_bo = gem_handle_to_libdrm_bo(bufmgr, drm_fd, "imported", fb_info->gem_handle);
+	igt_assert(fb_bo);
 
 	for (i = 0; i < limit; i++) {
-		BLIT_COPY_BATCH_START(devid, 0);
-		OUT_BATCH((3 << 24) | /* 32 bits */
-			  (0xcc << 16) | /* copy ROP */
-			  pitch);
-		OUT_BATCH(0 << 16 | 0);
-		OUT_BATCH(o->fb_height << 16 | o->fb_width);
-		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-		BLIT_RELOC_UDW(devid);
-		OUT_BATCH(0 << 16 | 0);
-		OUT_BATCH(pitch);
-		OUT_RELOC_FENCED(target_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
-		BLIT_RELOC_UDW(devid);
-		ADVANCE_BATCH();
+		blit_copy(dst_bo, src_bo,
+			  2048, 2048,
+			  2048*4, 2048*4);
 
-		if (IS_GEN6(devid) || IS_GEN7(devid)) {
-			BEGIN_BATCH(3);
-			OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
-			OUT_BATCH(0);
-			OUT_BATCH(0);
-			ADVANCE_BATCH();
-		}
-
-		tmp_bo = dummy_bo;
-		dummy_bo = target_bo;
-		target_bo = tmp_bo;
+		igt_swap(src_bo, dst_bo);
 	}
+	blit_copy(fb_bo, src_bo,
+		  min(o->fb_width, 2048), min(o->fb_height, 2048),
+		  fb_info->stride, 2048*4);
 	intel_batchbuffer_flush(batch);
 
-	drm_intel_bo_unreference(dummy_bo);
-	drm_intel_bo_unreference(target_bo);
+	if (timeout > 0)
+		ret = drm_intel_gem_bo_wait(fb_bo, timeout * NSEC_PER_SEC);
+
+	drm_intel_bo_unreference(src_bo);
+	drm_intel_bo_unreference(dst_bo);
+	drm_intel_bo_unreference(fb_bo);
+
+	return ret;
+}
+
+static void emit_dummy_load__bcs(struct test_output *o, int seconds)
+{
+	static int ops_per_sec;
+
+	if (ops_per_sec == 0)
+		ops_per_sec = calibrate_dummy_load(o, "bcs",
+						   _emit_dummy_load__bcs);
+
+	_emit_dummy_load__bcs(o, seconds * ops_per_sec, 0);
 }
 
 static void emit_fence_stress(struct test_output *o)
 {
 	const int num_fences = gem_available_fences(drm_fd);
-	struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
+	struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 *exec;
 	uint32_t buf[2] = { MI_BATCH_BUFFER_END, 0 };
@@ -218,7 +286,7 @@ static void emit_fence_stress(struct test_output *o)
 		uint32_t tiling = I915_TILING_X;
 		unsigned long pitch = 0;
 		bo[i] = drm_intel_bo_alloc_tiled(bufmgr,
-						 "tiled bo", 1024, 1024, 4,
+						 "X tiled bo", 1024, 1024, 4,
 						 &tiling, &pitch, 0);
 		exec[i].handle = bo[i]->handle;
 		exec[i].flags = EXEC_OBJECT_NEEDS_FENCE;
@@ -244,82 +312,78 @@ static void emit_fence_stress(struct test_output *o)
 	free(exec);
 }
 
-static void emit_dummy_load__rcs(struct test_output *o)
+static int _emit_dummy_load__rcs(struct test_output *o, int limit, int timeout)
 {
-	const struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
-	render_copyfunc_t copyfunc;
-	struct scratch_buf sb[2], *src, *dst;
-	int i, limit;
+	const struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
+	igt_render_copyfunc_t copyfunc;
+	struct igt_buf sb[3], *src, *dst, *fb;
+	int i, ret = 0;
 
-	copyfunc = get_render_copyfunc(devid);
+	copyfunc = igt_get_render_copyfunc(devid);
 	if (copyfunc == NULL)
-		return emit_dummy_load__bcs(o);
+		return _emit_dummy_load__bcs(o, limit, timeout);
 
-	limit = intel_gen(devid) < 6 ? 500 : 5000;
-
-	sb[0].bo = drm_intel_bo_alloc(bufmgr, "dummy_bo", fb_info->size, 4096);
+	sb[0].bo = drm_intel_bo_alloc(bufmgr, "dummy_bo", 2048*2048*4, 4096);
 	igt_assert(sb[0].bo);
 	sb[0].size = sb[0].bo->size;
 	sb[0].tiling = I915_TILING_NONE;
 	sb[0].data = NULL;
 	sb[0].num_tiles = sb[0].bo->size;
-	sb[0].stride = 4 * o->fb_width;
+	sb[0].stride = 4 * 2048;
 
-	sb[1].bo = gem_handle_to_libdrm_bo(bufmgr, drm_fd, "imported", fb_info->gem_handle);
+	sb[1].bo = drm_intel_bo_alloc(bufmgr, "dummy_bo", 2048*2048*4, 4096);
 	igt_assert(sb[1].bo);
 	sb[1].size = sb[1].bo->size;
-	sb[1].tiling = fb_info->tiling;
+	sb[1].tiling = I915_TILING_NONE;
 	sb[1].data = NULL;
 	sb[1].num_tiles = sb[1].bo->size;
-	sb[1].stride = fb_info->stride;
+	sb[1].stride = 4 * 2048;
+
+	sb[2].bo = gem_handle_to_libdrm_bo(bufmgr, drm_fd, "imported", fb_info->gem_handle);
+	igt_assert(sb[2].bo);
+	sb[2].size = sb[2].bo->size;
+	sb[2].tiling = fb_info->tiling;
+	sb[2].data = NULL;
+	sb[2].num_tiles = sb[2].bo->size;
+	sb[2].stride = fb_info->stride;
 
 	src = &sb[0];
 	dst = &sb[1];
+	fb = &sb[2];
 
 	for (i = 0; i < limit; i++) {
-		struct scratch_buf *tmp;
-
 		copyfunc(batch, NULL,
 			 src, 0, 0,
-			 o->fb_width, o->fb_height,
+			 2048, 2048,
 			 dst, 0, 0);
 
-		tmp = src;
-		src = dst;
-		dst = tmp;
+		igt_swap(src, dst);
 	}
+	copyfunc(batch, NULL,
+		 src, 0, 0,
+		 min(o->fb_width, 2048), min(o->fb_height, 2048),
+		 fb, 0, 0);
 	intel_batchbuffer_flush(batch);
 
+	if (timeout > 0)
+		ret = drm_intel_gem_bo_wait(fb->bo, timeout * NSEC_PER_SEC);
+
 	drm_intel_bo_unreference(sb[0].bo);
 	drm_intel_bo_unreference(sb[1].bo);
+	drm_intel_bo_unreference(sb[2].bo);
+
+	return ret;
 }
 
-static void set_connector_dpms(drmModeConnector *connector, int mode)
+static void emit_dummy_load__rcs(struct test_output *o, int seconds)
 {
-	int i, dpms = 0;
-	bool found_it = false;
+	static int ops_per_sec;
 
-	for (i = 0; i < connector->count_props; i++) {
-		struct drm_mode_get_property prop;
+	if (ops_per_sec == 0)
+		ops_per_sec = calibrate_dummy_load(o, "rcs",
+						   _emit_dummy_load__rcs);
 
-		prop.prop_id = connector->props[i];
-		prop.count_values = 0;
-		prop.count_enum_blobs = 0;
-		if (drmIoctl(drm_fd, DRM_IOCTL_MODE_GETPROPERTY, &prop))
-			continue;
-
-		if (strcmp(prop.name, "DPMS"))
-			continue;
-
-		dpms = prop.prop_id;
-		found_it = true;
-		break;
-	}
-	igt_assert_f(found_it, "DPMS property not found on %d\n",
-		     connector->connector_id);
-
-	igt_assert(drmModeConnectorSetProperty(drm_fd, connector->connector_id,
-					       dpms, mode) == 0);
+	_emit_dummy_load__bcs(o, seconds * ops_per_sec, 0);
 }
 
 static void dpms_off_other_outputs(struct test_output *o)
@@ -338,8 +402,8 @@ static void dpms_off_other_outputs(struct test_output *o)
 
 		connector = drmModeGetConnector(drm_fd, connector_id);
 
-		set_connector_dpms(connector,  DRM_MODE_DPMS_ON);
-		set_connector_dpms(connector,  DRM_MODE_DPMS_OFF);
+		kmstest_set_connector_dpms(drm_fd, connector,  DRM_MODE_DPMS_ON);
+		kmstest_set_connector_dpms(drm_fd, connector,  DRM_MODE_DPMS_OFF);
 
 		drmModeFreeConnector(connector);
 next:
@@ -350,7 +414,7 @@ next:
 static void set_dpms(struct test_output *o, int mode)
 {
 	for (int n = 0; n < o->count; n++)
-		set_connector_dpms(o->kconnector[n], mode);
+		kmstest_set_connector_dpms(drm_fd, o->kconnector[n], mode);
 }
 
 static void set_flag(unsigned int *v, unsigned int flag)
@@ -558,54 +622,59 @@ static void check_state(struct test_output *o, struct event_state *es)
 	double usec_interflip;
 
 	timersub(&es->current_ts, &es->current_received_ts, &diff);
-	if ((!analog_tv_connector(o)) &&
-	    (diff.tv_sec > 0 || (diff.tv_sec == 0 && diff.tv_usec > 2000))) {
-		fprintf(stderr, "%s ts delayed for too long: %is, %iusec\n",
-			es->name, (int)diff.tv_sec, (int)diff.tv_usec);
-		igt_fail(5);
+	if (!analog_tv_connector(o)) {
+		igt_assert_f(diff.tv_sec < 0 || (diff.tv_sec == 0 && diff.tv_usec <= 2000),
+			     "%s ts delayed for too long: %is, %iusec\n",
+			     es->name, (int)diff.tv_sec, (int)diff.tv_usec);
+
 	}
 
 	if (es->count == 0)
 		return;
 
-	if (!timercmp(&es->last_received_ts, &es->current_ts, <)) {
-		fprintf(stderr, "%s ts before the %s was issued!\n",
-				es->name, es->name);
+	timersub(&es->current_ts, &es->last_received_ts, &diff);
+	igt_assert_f(timercmp(&es->last_received_ts, &es->current_ts, <),
+		     "%s ts before the %s was issued!\n"
+		     "timerdiff %is, %ius\n",
+		     es->name, es->name,
+		     (int) diff.tv_sec, (int) diff.tv_usec);
 
-		timersub(&es->current_ts, &es->last_received_ts, &diff);
-		fprintf(stderr, "timerdiff %is, %ius\n",
-			(int) diff.tv_sec, (int) diff.tv_usec);
-		igt_fail(6);
-	}
+	/* check only valid if no modeset happens in between, that increments by
+	 * (1 << 23) on each step. This bounding matches the one in
+	 * DRM_IOCTL_WAIT_VBLANK. */
+	if (!(o->flags & (TEST_DPMS | TEST_MODESET)))
+		igt_assert_f(es->current_seq - (es->last_seq + o->seq_step) <= 1UL << 23,
+			     "unexpected %s seq %u, should be >= %u\n",
+			     es->name, es->current_seq, es->last_seq + o->seq_step);
 
-	/* This bounding matches the one in DRM_IOCTL_WAIT_VBLANK. */
-	if (!(o->flags & (TEST_DPMS | TEST_MODESET))) {
-		/* check only valid if no modeset happens in between, that
-		 * increments by (1 << 23) on each step. */
-		if (es->current_seq - (es->last_seq + o->seq_step) > 1UL << 23) {
-			fprintf(stderr, "unexpected %s seq %u, should be >= %u\n",
-				es->name, es->current_seq, es->last_seq + o->seq_step);
-			igt_fail(10);
-		}
+	/* Check that the vblank frame didn't wrap unexpectedly. */
+	if (o->flags & TEST_TS_CONT) {
+		/* Ignore seq_step here since vblank waits time out immediately
+		 * when we kill the crtc. */
+		igt_assert_f(es->current_seq - es->last_seq >= 0,
+			     "unexpected %s seq %u, should be >= %u\n",
+			     es->name, es->current_seq, es->last_seq);
+		igt_assert_f(es->current_seq - es->last_seq <= 100,
+			     "unexpected %s seq %u, should be < %u\n",
+			     es->name, es->current_seq, es->last_seq + 100);
+
+		igt_debug("testing ts continuity: Current frame %u, old frame %u\n",
+			  es->current_seq, es->last_seq);
 	}
 
 	if ((o->flags & TEST_CHECK_TS) && (!analog_tv_connector(o))) {
 		timersub(&es->current_ts, &es->last_ts, &diff);
 		usec_interflip = (double)o->seq_step * frame_time(o);
-		if (fabs((((double) diff.tv_usec) - usec_interflip) /
-		    usec_interflip) > 0.005) {
-			fprintf(stderr, "inter-%s ts jitter: %is, %ius\n",
-				es->name,
-				(int) diff.tv_sec, (int) diff.tv_usec);
-			igt_fail(9);
-		}
 
-		if (es->current_seq != es->last_seq + o->seq_step) {
-			fprintf(stderr, "unexpected %s seq %u, expected %u\n",
-					es->name, es->current_seq,
-					es->last_seq + o->seq_step);
-			igt_fail(9);
-		}
+		igt_assert_f(fabs((((double) diff.tv_usec) - usec_interflip) /
+				  usec_interflip) <= 0.005,
+			     "inter-%s ts jitter: %is, %ius\n",
+			     es->name, (int) diff.tv_sec, (int) diff.tv_usec);
+
+		igt_assert_f(es->current_seq == es->last_seq + o->seq_step,
+			     "unexpected %s seq %u, expected %u\n",
+			     es->name, es->current_seq,
+			     es->last_seq + o->seq_step);
 	}
 }
 
@@ -654,7 +723,7 @@ static void check_all_state(struct test_output *o,
 static void recreate_fb(struct test_output *o)
 {
 	drmModeFBPtr r;
-	struct kmstest_fb *fb_info = &o->fb_info[o->current_fb_id];
+	struct igt_fb *fb_info = &o->fb_info[o->current_fb_id];
 	uint32_t new_fb_id;
 
 	/* Call rmfb/getfb/addfb to ensure those don't introduce stalls */
@@ -676,7 +745,7 @@ static void recreate_fb(struct test_output *o)
 static void set_y_tiling(struct test_output *o, int fb_idx)
 {
 	drmModeFBPtr r;
-	struct kmstest_fb *fb_info = &o->fb_info[fb_idx];
+	struct igt_fb *fb_info = &o->fb_info[fb_idx];
 
 	/* Call rmfb/getfb/addfb to ensure those don't introduce stalls */
 	r = drmModeGetFB(drm_fd, fb_info->fb_id);
@@ -689,63 +758,29 @@ static void set_y_tiling(struct test_output *o, int fb_idx)
 
 static void stop_rings(bool stop)
 {
-	static const char dfs_base[] = "/sys/kernel/debug/dri";
-	static const char dfs_entry[] = "i915_ring_stop";
-	static const char stop_data[] = "0xf";
-	static const char run_data[] = "0x0";
-	char fname[FILENAME_MAX];
-	int card_index = drm_get_card();
-	int fd;
-
-	snprintf(fname, FILENAME_MAX, "%s/%i/%s",
-		 dfs_base, card_index, dfs_entry);
-
-	fd = open(fname, O_WRONLY);
-	igt_assert(fd >= 0);
-
 	if (stop)
-		igt_assert(write(fd, stop_data, sizeof(stop_data)) == sizeof(stop_data));
+		igt_set_stop_rings(STOP_RING_DEFAULTS);
 	else
-		igt_assert(write(fd, run_data, sizeof(run_data)) == sizeof(run_data));
-
-	close(fd);
+		igt_set_stop_rings(STOP_RING_NONE);
 }
 
 static void eat_error_state(void)
 {
-	static const char dfs_base[] = "/sys/kernel/debug/dri";
 	static const char dfs_entry_error[] = "i915_error_state";
-	static const char dfs_entry_stop[] = "i915_ring_stop";
 	static const char data[] = "";
-	static char tmp[128];
-	char fname[FILENAME_MAX];
-	int card_index = drm_get_card();
 	int fd;
 
-	igt_assert(card_index != -1);
-
-	/* clear the error state */
-	snprintf(fname, FILENAME_MAX, "%s/%i/%s",
-		 dfs_base, card_index, dfs_entry_error);
-
-	fd = open(fname, O_WRONLY);
-	igt_assert(fd >= 0);
+	fd = igt_debugfs_open(dfs_entry_error, O_WRONLY);
+	igt_assert_lte(0, fd);
 
 	igt_assert(write(fd, data, sizeof(data)) == sizeof(data));
 	close(fd);
 
 	/* and check whether stop_rings is not reset, i.e. the hang has indeed
 	 * happened */
-	snprintf(fname, FILENAME_MAX, "%s/%i/%s",
-		 dfs_base, card_index, dfs_entry_stop);
-
-	fd = open(fname, O_RDONLY);
-	igt_assert(fd >= 0);
-
-	igt_assert(read(fd, tmp, sizeof tmp) > 0);
-
-	igt_assert_f(atoi(tmp) == 0,
-		     "no gpu hang detected, stop_rings is still %s\n", tmp);
+	igt_assert_f(igt_get_stop_rings() == STOP_RING_NONE,
+		     "no gpu hang detected, stop_rings is still 0x%x\n",
+		     igt_get_stop_rings());
 
 	close(fd);
 }
@@ -847,15 +882,15 @@ static unsigned int run_test_step(struct test_output *o)
 	if (o->flags & TEST_DPMS_OFF_OTHERS)
 		dpms_off_other_outputs(o);
 
-	if (o->flags & TEST_WITH_DUMMY_BCS)
-		emit_dummy_load__bcs(o);
-
-	if (o->flags & TEST_WITH_DUMMY_RCS)
-		emit_dummy_load__rcs(o);
-
 	if (!(o->flags & TEST_SINGLE_BUFFER))
 		o->current_fb_id = !o->current_fb_id;
 
+	if (o->flags & TEST_WITH_DUMMY_BCS)
+		emit_dummy_load__bcs(o, 1);
+
+	if (o->flags & TEST_WITH_DUMMY_RCS)
+		emit_dummy_load__rcs(o, 1);
+
 	if (o->flags & TEST_FB_RECREATE)
 		recreate_fb(o);
 	new_fb_id = o->fb_ids[o->current_fb_id];
@@ -919,20 +954,32 @@ static unsigned int run_test_step(struct test_output *o)
 
 		/* modeset/DPMS is done, vblank wait should work normally now */
 		start = gettime_us();
-		igt_assert(__wait_for_vblank(TEST_VBLANK_BLOCK, o->pipe, 1, 0, &reply) == 0);
+		igt_assert(__wait_for_vblank(TEST_VBLANK_BLOCK, o->pipe, 2, 0, &reply) == 0);
 		end = gettime_us();
-		igt_assert(end - start > 1 * frame_time(o) / 2 &&
-			   end - start < 3 * frame_time(o) / 2);
+		/*
+		 * we waited for two vblanks, so verify that
+		 * we were blocked for ~1-2 frames.
+		 */
+		igt_assert_f(end - start > 0.9 * frame_time(o) &&
+			     end - start < 2.1 * frame_time(o),
+			     "wait for two vblanks took %lu usec (frame time %f usec)\n",
+			     end - start, frame_time(o));
 		join_vblank_wait_thread();
 	}
 
-	igt_info("."); fflush(stdout);
+	igt_print_activity();
 
 	if (do_flip && (o->flags & TEST_HANG)) {
 		hang = hang_gpu(drm_fd);
 		igt_assert_f(hang, "failed to exercise page flip hang recovery\n");
 	}
 
+	/* try to make sure we can issue two flips during the same frame */
+	if (do_flip && (o->flags & TEST_EBUSY)) {
+		struct vblank_reply reply;
+		igt_assert(__wait_for_vblank(TEST_VBLANK_BLOCK, o->pipe, 1, 0, &reply) == 0);
+	}
+
 	if (do_flip)
 		do_or_die(do_page_flip(o, new_fb_id, !(o->flags & TEST_NOEVENT)));
 
@@ -960,7 +1007,15 @@ static unsigned int run_test_step(struct test_output *o)
 	if (o->flags & TEST_PAN) {
 		int count = do_flip ?
 			o->flip_state.count : o->vblank_state.count;
-		int x_ofs = count * 10 > o->fb_width - o->kmode[0].hdisplay ? o->fb_width - o->kmode[0].hdisplay : count * 10;
+		int x_ofs = min(count * 10, o->fb_width - o->kmode[0].hdisplay);
+
+		/* Make sure DSPSURF changes value */
+		if (o->flags & TEST_HANG)
+			o->current_fb_id = !o->current_fb_id;
+
+		/* Make sure DSPSURF changes value */
+		if (o->flags & TEST_HANG)
+			o->current_fb_id = !o->current_fb_id;
 
 		igt_assert_f(set_mode(o, o->fb_ids[o->current_fb_id], x_ofs, 0) == 0,
 			     "failed to pan (%dx%d@%dHz)+%d: %s\n",
@@ -976,6 +1031,12 @@ static unsigned int run_test_step(struct test_output *o)
 			     "failed to disable output: %s\n",
 			     strerror(errno));
 
+	if (o->flags & TEST_RPM)
+		igt_assert(igt_wait_for_pm_status(IGT_RUNTIME_PM_STATUS_SUSPENDED));
+
+	if (o->flags & TEST_SUSPEND)
+		igt_system_suspend_autoresume();
+
 	if (do_vblank && (o->flags & TEST_EINVAL) && o->vblank_state.count > 0)
 		igt_assert(do_wait_for_vblank(o, o->pipe, target_seq, &vbl_reply)
 			   == -EINVAL);
@@ -1012,8 +1073,8 @@ static void connector_find_preferred_mode(uint32_t connector_id, int crtc_idx,
 {
 	struct kmstest_connector_config config;
 
-	if (kmstest_get_connector_config(drm_fd, connector_id, 1 << crtc_idx,
-					 &config) < 0) {
+	if (!kmstest_get_connector_config(drm_fd, connector_id, 1 << crtc_idx,
+					  &config)) {
 		o->mode_valid = 0;
 		return;
 	}
@@ -1055,12 +1116,12 @@ static void connector_find_compatible_mode(int crtc_idx0, int crtc_idx1,
 	drmModeModeInfo *mode[2];
 	int n, m;
 
-	if (kmstest_get_connector_config(drm_fd, o->_connector[0],
-					 1 << crtc_idx0, &config[0]) < 0)
+	if (!kmstest_get_connector_config(drm_fd, o->_connector[0],
+					  1 << crtc_idx0, &config[0]))
 		return;
 
-	if (kmstest_get_connector_config(drm_fd, o->_connector[1],
-					 1 << crtc_idx1, &config[1]) < 0) {
+	if (!kmstest_get_connector_config(drm_fd, o->_connector[1],
+					  1 << crtc_idx1, &config[1])) {
 		kmstest_free_connector_config(&config[0]);
 		return;
 	}
@@ -1101,13 +1162,13 @@ found:
 	drmModeFreeCrtc(config[1].crtc);
 }
 
-static void paint_flip_mode(struct kmstest_fb *fb, bool odd_frame)
+static void paint_flip_mode(struct igt_fb *fb, bool odd_frame)
 {
-	cairo_t *cr = kmstest_get_cairo_ctx(drm_fd, fb);
+	cairo_t *cr = igt_get_cairo_ctx(drm_fd, fb);
 	int width = fb->width;
 	int height = fb->height;
 
-	kmstest_paint_test_pattern(cr, width, height);
+	igt_paint_test_pattern(cr, width, height);
 
 	if (odd_frame)
 		cairo_rectangle(cr, width/4, height/2, width/4, height/8);
@@ -1211,6 +1272,13 @@ static unsigned int wait_for_events(struct test_output *o)
 static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 {
 	unsigned long start, end;
+	uint32_t hang = 0;	/* Suppress GCC warning */
+	int count = 0;
+
+	if (o->flags & TEST_HANG_ONCE) {
+		hang = hang_gpu(drm_fd);
+		igt_assert_f(hang, "failed to exercise page flip hang recovery\n");
+	}
 
 	start = gettime_us();
 
@@ -1223,12 +1291,17 @@ static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 		check_all_state(o, completed_events);
 		update_all_state(o, completed_events);
 
-		if ((gettime_us() - start) / 1000 >= duration_ms)
+		if (count && (gettime_us() - start) / 1000 >= duration_ms)
 			break;
+
+		count++;
 	}
 
 	end = gettime_us();
 
+	if (hang)
+		unhang_gpu(drm_fd, hang);
+
 	/* Flush any remaining events */
 	if (o->pending_events)
 		wait_for_events(o);
@@ -1251,7 +1324,8 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 {
 	char test_name[128];
 	unsigned elapsed;
-	bool tiled;
+	unsigned bo_size = 0;
+	uint64_t tiling;
 	int i;
 
 	switch (crtc_count) {
@@ -1274,7 +1348,7 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	if (!o->mode_valid)
 		return;
 
-	igt_assert(o->count == crtc_count);
+	igt_assert_eq(o->count, crtc_count);
 
 	last_connector = o->kconnector[0];
 
@@ -1283,23 +1357,34 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	if (o->flags & TEST_PAN)
 		o->fb_width *= 2;
 
-	tiled = false;
+	tiling = LOCAL_DRM_FORMAT_MOD_NONE;
 	if (o->flags & TEST_FENCE_STRESS)
-		tiled = true;
+		tiling = LOCAL_I915_FORMAT_MOD_X_TILED;
 
-	o->fb_ids[0] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, tiled, &o->fb_info[0]);
-	o->fb_ids[1] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, tiled, &o->fb_info[1]);
-	o->fb_ids[2] = kmstest_create_fb(drm_fd, o->fb_width, o->fb_height,
-					 o->bpp, o->depth, true, &o->fb_info[2]);
+	/* 256 MB is usually the maximum mappable aperture,
+	 * (make it 4x times that to ensure failure) */
+	if (o->flags & TEST_BO_TOOBIG)
+		bo_size = 4*256*1024*1024;
+
+	o->fb_ids[0] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
+					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
+					 tiling, &o->fb_info[0]);
+	o->fb_ids[1] = igt_create_fb_with_bo_size(drm_fd, o->fb_width, o->fb_height,
+					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
+					 tiling, &o->fb_info[1], bo_size);
+	o->fb_ids[2] = igt_create_fb(drm_fd, o->fb_width, o->fb_height,
+					 igt_bpp_depth_to_drm_format(o->bpp, o->depth),
+					 LOCAL_I915_FORMAT_MOD_X_TILED, &o->fb_info[2]);
 	igt_assert(o->fb_ids[0]);
 	igt_assert(o->fb_ids[1]);
-	igt_assert(o->fb_ids[2]);
+	if (o->flags & TEST_FB_BAD_TILING)
+		igt_require(o->fb_ids[2]);
 
 	paint_flip_mode(&o->fb_info[0], false);
-	paint_flip_mode(&o->fb_info[1], true);
-	paint_flip_mode(&o->fb_info[2], true);
+	if (!(o->flags & TEST_BO_TOOBIG))
+		paint_flip_mode(&o->fb_info[1], true);
+	if (o->fb_ids[2])
+		paint_flip_mode(&o->fb_info[2], true);
 
 	if (o->flags & TEST_FB_BAD_TILING)
 		set_y_tiling(o, 2);
@@ -1321,10 +1406,15 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	if (o->flags & TEST_CHECK_TS)
 		sleep(1);
 
-	igt_assert(do_page_flip(o, o->fb_ids[1], true) == 0);
+	if (o->flags & TEST_BO_TOOBIG) {
+		igt_assert_eq(do_page_flip(o, o->fb_ids[1], true), -E2BIG);
+		goto out;
+	} else
+		igt_assert_eq(do_page_flip(o, o->fb_ids[1], true), 0);
 	wait_for_events(o);
 
 	o->current_fb_id = 1;
+
 	if (o->flags & TEST_FLIP)
 		o->flip_state.seq_step = 1;
 	else
@@ -1347,9 +1437,10 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	igt_info("\n%s: PASSED\n\n", test_name);
 
 out:
-	kmstest_remove_fb(drm_fd, &o->fb_info[2]);
-	kmstest_remove_fb(drm_fd, &o->fb_info[1]);
-	kmstest_remove_fb(drm_fd, &o->fb_info[0]);
+	if (o->fb_ids[2])
+		igt_remove_fb(drm_fd, &o->fb_info[2]);
+	igt_remove_fb(drm_fd, &o->fb_info[1]);
+	igt_remove_fb(drm_fd, &o->fb_info[0]);
 
 	last_connector = NULL;
 
@@ -1363,6 +1454,9 @@ static int run_test(int duration, int flags)
 
 	igt_require((flags & TEST_HANG) == 0 || !is_hung(drm_fd));
 
+	if (flags & TEST_RPM)
+		igt_require(igt_setup_runtime_pm());
+
 	resources = drmModeGetResources(drm_fd);
 	igt_assert(resources);
 
@@ -1386,9 +1480,9 @@ static int run_test(int duration, int flags)
 		}
 	}
 
-	igt_assert(modes);
+	igt_require(modes);
 	duration = duration * 1000 / modes;
-	duration = duration < 500 ? 500 : duration;
+	duration = max(500, duration);
 
 	/* Find any connected displays */
 	for (i = 0; i < resources->count_connectors; i++) {
@@ -1450,9 +1544,9 @@ static int run_pair(int duration, int flags)
 
 	/* If we have fewer than 2 connected outputs then we won't have any
 	 * configuration at all. So skip in that case. */
-	igt_require(modes);
+	igt_require_f(modes, "At least two displays required\n");
 	duration = duration * 1000 / modes;
-	duration = duration < 500 ? 500 : duration;
+	duration = max(duration, 500);
 
 	/* Find a pair of connected displays */
 	for (i = 0; i < resources->count_connectors; i++) {
@@ -1501,10 +1595,34 @@ static void kms_flip_exit_handler(int sig)
 {
 	igt_fixture {
 		if (last_connector)
-			set_connector_dpms(last_connector, DRM_MODE_DPMS_ON);
+			kmstest_set_connector_dpms(drm_fd, last_connector, DRM_MODE_DPMS_ON);
 	}
 }
 
+static void test_nonblocking_read(int in)
+{
+	char buffer[1024];
+	int fd = dup(in);
+	int ret;
+
+	ret = -1;
+	if (fd != -1)
+		ret = fcntl(fd, F_GETFL);
+	if (ret != -1) {
+		ret |= O_NONBLOCK;
+		ret = fcntl(fd, F_SETFL, ret);
+	}
+	igt_require(ret != -1);
+
+	igt_set_timeout(5);
+	ret = read(fd, buffer, sizeof(buffer));
+	igt_set_timeout(0);
+	igt_assert_eq(ret, -1);
+	igt_assert_eq(errno, EAGAIN);
+
+	close(fd);
+}
+
 int main(int argc, char **argv)
 {
 	struct {
@@ -1522,30 +1640,31 @@ int main(int argc, char **argv)
 					"blocking-absolute-wf_vblank" },
 		{ 60,  TEST_VBLANK | TEST_DPMS | TEST_EINVAL, "wf_vblank-vs-dpms" },
 		{ 60,  TEST_VBLANK | TEST_DPMS | TEST_WITH_DUMMY_BCS,
-					"bcs-wf_vblank-vs-dpms" },
+					"blt-wf_vblank-vs-dpms" },
 		{ 60,  TEST_VBLANK | TEST_DPMS | TEST_WITH_DUMMY_RCS,
 					"rcs-wf_vblank-vs-dpms" },
 		{ 60,  TEST_VBLANK | TEST_MODESET | TEST_EINVAL, "wf_vblank-vs-modeset" },
 		{ 60,  TEST_VBLANK | TEST_MODESET | TEST_WITH_DUMMY_BCS,
-					"bcs-wf_vblank-vs-modeset" },
+					"blt-wf_vblank-vs-modeset" },
 		{ 60,  TEST_VBLANK | TEST_MODESET | TEST_WITH_DUMMY_RCS,
 					"rcs-wf_vblank-vs-modeset" },
 
-		{ 30, TEST_FLIP | TEST_EBUSY , "plain-flip" },
+		{ 30, TEST_FLIP , "plain-flip" },
+		{ 30, TEST_FLIP | TEST_EBUSY , "busy-flip" },
 		{ 30, TEST_FLIP | TEST_FENCE_STRESS , "flip-vs-fences" },
-		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_EBUSY , "plain-flip-ts-check" },
-		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_EBUSY | TEST_FB_RECREATE,
+		{ 30, TEST_FLIP | TEST_CHECK_TS, "plain-flip-ts-check" },
+		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_FB_RECREATE,
 			"plain-flip-fb-recreate" },
-		{ 30, TEST_FLIP | TEST_EBUSY | TEST_RMFB | TEST_MODESET , "flip-vs-rmfb" },
+		{ 30, TEST_FLIP | TEST_RMFB | TEST_MODESET , "flip-vs-rmfb" },
 		{ 60, TEST_FLIP | TEST_DPMS | TEST_EINVAL, "flip-vs-dpms" },
-		{ 60, TEST_FLIP | TEST_DPMS | TEST_WITH_DUMMY_BCS, "bcs-flip-vs-dpms" },
-		{ 60, TEST_FLIP | TEST_DPMS | TEST_WITH_DUMMY_RCS, "rcs-flip-vs-dpms" },
+		{ 60, TEST_FLIP | TEST_DPMS | TEST_WITH_DUMMY_BCS, "blt-flip-vs-dpms" },
+		{ 60, TEST_FLIP | TEST_DPMS | TEST_WITH_DUMMY_RCS, "render-flip-vs-dpms" },
 		{ 30,  TEST_FLIP | TEST_PAN, "flip-vs-panning" },
-		{ 60, TEST_FLIP | TEST_PAN | TEST_WITH_DUMMY_BCS, "bcs-flip-vs-panning" },
-		{ 60, TEST_FLIP | TEST_PAN | TEST_WITH_DUMMY_RCS, "rcs-flip-vs-panning" },
+		{ 60, TEST_FLIP | TEST_PAN | TEST_WITH_DUMMY_BCS, "blt-flip-vs-panning" },
+		{ 60, TEST_FLIP | TEST_PAN | TEST_WITH_DUMMY_RCS, "render-flip-vs-panning" },
 		{ 60, TEST_FLIP | TEST_MODESET | TEST_EINVAL, "flip-vs-modeset" },
-		{ 60, TEST_FLIP | TEST_MODESET | TEST_WITH_DUMMY_BCS, "bcs-flip-vs-modeset" },
-		{ 60, TEST_FLIP | TEST_MODESET | TEST_WITH_DUMMY_RCS, "rcs-flip-vs-modeset" },
+		{ 60, TEST_FLIP | TEST_MODESET | TEST_WITH_DUMMY_BCS, "blt-flip-vs-modeset" },
+		{ 60, TEST_FLIP | TEST_MODESET | TEST_WITH_DUMMY_RCS, "render-flip-vs-modeset" },
 		{ 30,  TEST_FLIP | TEST_VBLANK_EXPIRED_SEQ,
 					"flip-vs-expired-vblank" },
 
@@ -1557,6 +1676,7 @@ int main(int argc, char **argv)
 			TEST_CHECK_TS, "flip-vs-blocking-wf-vblank" },
 		{ 30, TEST_FLIP | TEST_MODESET | TEST_HANG | TEST_NOEVENT, "flip-vs-modeset-vs-hang" },
 		{ 30, TEST_FLIP | TEST_PAN | TEST_HANG, "flip-vs-panning-vs-hang" },
+		{ 30, TEST_VBLANK | TEST_HANG_ONCE, "vblank-vs-hang" },
 		{ 1, TEST_FLIP | TEST_EINVAL | TEST_FB_BAD_TILING, "flip-vs-bad-tiling" },
 
 		{ 1, TEST_DPMS_OFF | TEST_MODESET | TEST_FLIP,
@@ -1567,6 +1687,12 @@ int main(int argc, char **argv)
 		{ 0, TEST_ENOENT | TEST_NOEVENT, "nonexisting-fb" },
 		{ 10, TEST_DPMS_OFF | TEST_DPMS | TEST_VBLANK_RACE, "dpms-vs-vblank-race" },
 		{ 10, TEST_MODESET | TEST_VBLANK_RACE, "modeset-vs-vblank-race" },
+		{ 10, TEST_VBLANK | TEST_DPMS | TEST_RPM | TEST_TS_CONT, "vblank-vs-dpms-rpm" },
+		{ 10, TEST_VBLANK | TEST_MODESET | TEST_RPM | TEST_TS_CONT, "vblank-vs-modeset-rpm" },
+		{ 0, TEST_VBLANK | TEST_DPMS | TEST_SUSPEND | TEST_TS_CONT, "vblank-vs-dpms-suspend" },
+		{ 0, TEST_VBLANK | TEST_MODESET | TEST_SUSPEND | TEST_TS_CONT, "vblank-vs-modeset-suspend" },
+		{ 0, TEST_VBLANK | TEST_SUSPEND | TEST_TS_CONT, "vblank-vs-suspend" },
+		{ 0, TEST_BO_TOOBIG | TEST_NO_2X_OUTPUT, "bo-too-big" },
 	};
 	int i;
 
@@ -1574,9 +1700,11 @@ int main(int argc, char **argv)
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		drm_fd = drm_open_any();
+		drm_fd = drm_open_any_master();
 
-		igt_set_vt_graphics_mode();
+		igt_enable_connectors();
+
+		kmstest_set_vt_graphics_mode();
 		igt_install_exit_handler(kms_flip_exit_handler);
 		get_timestamp_format();
 
@@ -1585,6 +1713,9 @@ int main(int argc, char **argv)
 		batch = intel_batchbuffer_alloc(bufmgr, devid);
 	}
 
+	igt_subtest("nonblocking-read")
+		test_nonblocking_read(drm_fd);
+
 	for (i = 0; i < sizeof(tests) / sizeof (tests[0]); i++) {
 		igt_subtest(tests[i].name)
 			run_test(tests[i].duration, tests[i].flags);
@@ -1592,6 +1723,10 @@ int main(int argc, char **argv)
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
+		/* code doesn't disable all crtcs, so skip rpm tests */
+		if (tests[i].flags & TEST_RPM)
+			continue;
+
 		igt_subtest_f( "2x-%s", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
@@ -1610,6 +1745,10 @@ int main(int argc, char **argv)
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
+		/* code doesn't disable all crtcs, so skip rpm tests */
+		if (tests[i].flags & TEST_RPM)
+			continue;
+
 		igt_subtest_f( "2x-%s-interruptible", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
