@@ -112,12 +112,12 @@ static int test_format(const char *test_name,
 
 	ret = asprintf(&mode_format_str, "%s @ %dHz / %s",
 		 mode->name, mode->vrefresh, igt_format_str(format));
-	igt_assert(ret > 0);
+	igt_assert_lt(0, ret);
 	ret = asprintf(&cconf_str, "pipe %s, encoder %s, connector %s",
-		       kmstest_pipe_str(cconf->pipe),
+		       kmstest_pipe_name(cconf->pipe),
 		       kmstest_encoder_type_str(cconf->encoder->encoder_type),
 		       kmstest_connector_type_str(cconf->connector->connector_type));
-	igt_assert(ret > 0);
+	igt_assert_lt(0, ret);
 
 	igt_info("Beginning test %s with %s on %s\n",
 		 test_name, mode_format_str, cconf_str);
@@ -125,10 +125,12 @@ static int test_format(const char *test_name,
 	width = mode->hdisplay;
 	height = mode->vdisplay;
 
-	if (!igt_create_fb(drm_fd, width, height, format, false, &fb[0]))
+	if (!igt_create_fb(drm_fd, width, height, format,
+			   LOCAL_DRM_FORMAT_MOD_NONE, &fb[0]))
 		goto err1;
 
-	if (!igt_create_fb(drm_fd, width, height, format, false, &fb[1]))
+	if (!igt_create_fb(drm_fd, width, height, format,
+			   LOCAL_DRM_FORMAT_MOD_NONE,	&fb[1]))
 		goto err2;
 
 	if (drmModeSetCrtc(drm_fd, cconf->crtc->crtc_id, fb[0].fb_id,
@@ -177,10 +179,17 @@ static void test_connector(const char *test_name,
 	int i;
 
 	igt_get_all_formats(&formats, &format_count);
-	for (i = 0; i < format_count; i++)
+	for (i = 0; i < format_count; i++) {
+		if (intel_gen(intel_get_drm_devid(drm_fd)) < 4
+		    && formats[i] == DRM_FORMAT_XRGB2101010) {
+			igt_info("gen2/3 don't support 10bpc, skipping\n");
+			continue;
+		}
+
 		test_format(test_name,
-			    cconf, &cconf->connector->modes[i],
+			    cconf, &cconf->connector->modes[0],
 			    formats[i], flags);
+	}
 }
 
 static int run_test(const char *test_name, enum test_flags flags)
@@ -198,11 +207,9 @@ static int run_test(const char *test_name, enum test_flags flags)
 		connector_id = resources->connectors[i];
 		for (j = 0; j < resources->count_crtcs; j++) {
 			struct kmstest_connector_config cconf;
-			int ret;
 
-			ret = kmstest_get_connector_config(drm_fd, connector_id,
-							   1 << j, &cconf);
-			if (ret < 0)
+			if (!kmstest_get_connector_config(drm_fd, connector_id,
+							   1 << j, &cconf))
 				continue;
 
 			test_connector(test_name, &cconf, flags);
@@ -230,13 +237,13 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		drm_fd = drm_open_any();
+		drm_fd = drm_open_any_master();
 
 		bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
 		devid = intel_get_drm_devid(drm_fd);
 		batch = intel_batchbuffer_alloc(bufmgr, devid);
 
-		igt_set_vt_graphics_mode();
+		kmstest_set_vt_graphics_mode();
 	}
 
 	for (i = 0; i < ARRAY_SIZE(tests); i++) {
