@@ -49,7 +49,7 @@
 #include "ioctl_wrappers.h"
 #include "igt_aux.h"
 #include "igt_kms.h"
-#include "igt_debugfs.h"
+#include "igt_gt.h"
 
 /* One day, this will be on your libdrm. */
 #define DRM_CLIENT_CAP_UNIVERSAL_PLANES 2
@@ -285,7 +285,8 @@ static bool init_modeset_params_for_type(struct mode_set_data *data,
 		return false;
 
 	igt_create_fb(drm_fd, mode->hdisplay, mode->vdisplay,
-		      DRM_FORMAT_XRGB8888, I915_TILING_NONE, &params->fb);
+		      DRM_FORMAT_XRGB8888, LOCAL_DRM_FORMAT_MOD_NONE,
+		      &params->fb);
 	cr = igt_get_cairo_ctx(drm_fd, &params->fb);
 	igt_paint_test_pattern(cr, mode->hdisplay, mode->vdisplay);
 	cairo_destroy(cr);
@@ -459,7 +460,7 @@ static void free_drm_info(struct compare_data *data)
 	drmModeFreeResources(data->res);
 }
 
-#define COMPARE(d1, d2, data) igt_assert(d1->data == d2->data)
+#define COMPARE(d1, d2, data) igt_assert_eq(d1->data, d2->data)
 #define COMPARE_ARRAY(d1, d2, size, data) do { \
 	for (i = 0; i < size; i++) \
 		igt_assert(d1->data[i] == d2->data[i]); \
@@ -624,7 +625,7 @@ static int count_i2c_valid_edids(void)
 		if (strncmp(dirent->d_name, "i2c-", 4) == 0) {
 			snprintf(full_name, 32, "/dev/%s", dirent->d_name);
 			fd = open(full_name, O_RDWR);
-			igt_assert(fd != -1);
+			igt_assert_neq(fd, -1);
 			if (i2c_edid_is_valid(fd))
 				ret++;
 			close(fd);
@@ -636,12 +637,35 @@ static int count_i2c_valid_edids(void)
 	return ret;
 }
 
+static int count_vga_outputs(struct mode_set_data *data)
+{
+	int i, count = 0;
+
+	for (i = 0; i < data->res->count_connectors; i++)
+		if (data->connectors[i]->connector_type ==
+		    DRM_MODE_CONNECTOR_VGA)
+			count++;
+
+	return count;
+}
+
 static void test_i2c(struct mode_set_data *data)
 {
 	int i2c_edids = count_i2c_valid_edids();
 	int drm_edids = count_drm_valid_edids(data);
+	int vga_outputs = count_vga_outputs(data);
+	int diff;
 
-	igt_assert_eq(i2c_edids, drm_edids);
+	igt_debug("i2c edids:%d drm edids:%d vga outputs:%d\n",
+		  i2c_edids, drm_edids, vga_outputs);
+
+	/* We fail to detect some VGA monitors using our i2c method. If you look
+	 * at the dmesg of these cases, you'll see the Kernel complaining about
+	 * the EDID reading mostly FFs and then disabling bit-banging. Since we
+	 * don't want to reimplement everything the Kernel does, let's just
+	 * accept the fact that some VGA outputs won't be properly detected. */
+	diff = drm_edids - i2c_edids;
+	igt_assert(diff <= vga_outputs && diff >= 0);
 }
 
 static void setup_pc8(void)
@@ -663,8 +687,7 @@ static void setup_pc8(void)
 	if (!supports_pc8_plus_residencies())
 		return;
 
-	pc8_status_fd = open("/sys/kernel/debug/dri/0/i915_pc8_status",
-			     O_RDONLY);
+	pc8_status_fd = igt_debugfs_open("i915_pc8_status", O_RDONLY);
 	igt_assert_f(pc8_status_fd >= 0,
 		     "Can't open /sys/kernel/debug/dri/0/i915_pc8_status");
 
@@ -693,31 +716,15 @@ static void setup_non_graphics_runtime_pm(void)
 		if (fd < 0)
 			break;
 
-		write(fd, "min_power\n", 10);
+		igt_assert(write(fd, "min_power\n", 10) == 10);
 		close(fd);
 	}
 	free(file_name);
-
-	/* Audio runtime PM policies. */
-	fd = open("/sys/module/snd_hda_intel/parameters/power_save", O_WRONLY);
-	if (fd >= 0) {
-		write(fd, "1\n", 2);
-		close(fd);
-	}
-	fd = open("/sys/bus/pci/devices/0000:00:03.0/power/control", O_WRONLY);
-	if (fd >= 0) {
-		write(fd, "auto\n", 5);
-		close(fd);
-	}
 }
 
 static void setup_environment(void)
 {
-	drm_fd = drm_open_any();
-	igt_assert(drm_fd >= 0);
-
-	igt_require_f(drmSetMaster(drm_fd) == 0, "Can't become DRM master, "
-		      "please check if no other DRM client is running.\n");
+	drm_fd = drm_open_any_master();
 
 	init_mode_set_data(&ms_data);
 
@@ -772,6 +779,9 @@ static void modeset_subtest(enum screen_type type, int rounds, int wait_flags)
 	if (wait_flags & WAIT_PC8_RES)
 		igt_require(has_pc8);
 
+	if (wait_flags & WAIT_EXTRA)
+		rounds /= 2;
+
 	for (i = 0; i < rounds; i++) {
 		if (wait_flags & USE_DPMS)
 			disable_all_screens_dpms(&ms_data);
@@ -874,7 +884,7 @@ static void read_full_file(const char *name)
 	} while (rc == ARRAY_SIZE(buf));
 
 	rc = close(fd);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 
 	igt_assert_f(wait_for_suspended(), "File: %s\n", name);
 }
@@ -891,7 +901,7 @@ static void read_files_from_dir(const char *name, int level)
 
 	full_name = malloc(PATH_MAX);
 
-	igt_assert(level < 128);
+	igt_assert_lt(level, 128);
 
 	while ((dirent = readdir(dir))) {
 		struct stat stat_buf;
@@ -904,7 +914,7 @@ static void read_files_from_dir(const char *name, int level)
 		snprintf(full_name, PATH_MAX, "%s/%s", name, dirent->d_name);
 
 		rc = lstat(full_name, &stat_buf);
-		igt_assert(rc == 0);
+		igt_assert_eq(rc, 0);
 
 		if (S_ISDIR(stat_buf.st_mode))
 			read_files_from_dir(full_name, level + 1);
@@ -970,7 +980,7 @@ static void debugfs_forcewake_user_subtest(void)
 	}
 
 	rc = close(fd);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 
 	igt_assert(wait_for_suspended());
 }
@@ -991,7 +1001,7 @@ static void gem_mmap_subtest(bool gtt_mmap)
 		gem_buf = gem_mmap__gtt(drm_fd, handle, buf_size,
 					PROT_READ | PROT_WRITE);
 	else
-		gem_buf = gem_mmap__cpu(drm_fd, handle, buf_size, 0);
+		gem_buf = gem_mmap__cpu(drm_fd, handle, 0, buf_size, 0);
 
 
 	for (i = 0; i < buf_size; i++)
@@ -1026,7 +1036,7 @@ static void gem_mmap_subtest(bool gtt_mmap)
 		gem_buf = gem_mmap__gtt(drm_fd, handle, buf_size,
 					PROT_READ | PROT_WRITE);
 	else
-		gem_buf = gem_mmap__cpu(drm_fd, handle, buf_size, 0);
+		gem_buf = gem_mmap__cpu(drm_fd, handle, 0, buf_size, 0);
 
 	igt_assert(wait_for_suspended());
 
@@ -1224,7 +1234,7 @@ static void gem_execbuf_subtest(void)
 
 			if (y >= sq_y && y < (sq_y + sq_h) &&
 			    x >= sq_x && x < (sq_x + sq_w))
-				igt_assert(px == color);
+				igt_assert_eq_u32(px, color);
 			else
 				igt_assert(px == 0);
 		}
@@ -1241,7 +1251,7 @@ static void gem_execbuf_subtest(void)
 
 			if (y >= sq_y && y < (sq_y + sq_h) &&
 			    x >= sq_x && x < (sq_x + sq_w))
-				igt_assert(px == color);
+				igt_assert_eq_u32(px, color);
 			else
 				igt_assert(px == 0);
 		}
@@ -1264,7 +1274,7 @@ static void gem_execbuf_subtest(void)
 
 			if (y >= sq_y && y < (sq_y + sq_h) &&
 			    x >= sq_x && x < (sq_x + sq_w))
-				igt_assert(px == color);
+				igt_assert_eq_u32(px, color);
 			else
 				igt_assert(px == 0);
 		}
@@ -1333,6 +1343,29 @@ static void gem_idle_subtest(void)
 	gem_quiescent_gpu(drm_fd);
 }
 
+static void gem_evict_pwrite_subtest(void)
+{
+	static drm_intel_bufmgr *bufmgr;
+	uint32_t buf;
+	int i;
+
+	bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
+	igt_assert(bufmgr);
+	igt_init_aperture_trashers(bufmgr);
+
+	igt_trash_aperture();
+
+	disable_or_dpms_all_screens_and_wait(&ms_data, true);
+	igt_assert(wait_for_suspended());
+
+	buf = 0;
+	for (i = 0; i < num_trash_bos; i++)
+		gem_write(drm_fd, trash_bos[i]->handle, 0, &buf, sizeof(buf));
+
+	igt_cleanup_aperture_trashers();
+	drm_intel_bufmgr_destroy(bufmgr);
+}
+
 /* This also triggered WARNs on dmesg at some point. */
 static void reg_read_ioctl_subtest(void)
 {
@@ -1356,7 +1389,7 @@ static bool device_in_pci_d3(void)
 	pci_dev = intel_get_pci_device();
 
 	rc = pci_device_cfg_read_u16(pci_dev, &val, 0xd4);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 
 	return (val & 0x3) == 0x3;
 }
@@ -1389,6 +1422,55 @@ static void system_suspend_subtest(void)
 	igt_assert(wait_for_suspended());
 }
 
+static void system_suspend_execbuf_subtest(void)
+{
+	int i;
+	int batch_size = 4 * sizeof(uint32_t);
+	uint32_t batch_buf[batch_size];
+	uint32_t handle;
+	struct drm_i915_gem_execbuffer2 execbuf = {};
+	struct drm_i915_gem_exec_object2 objs[1] = {{}};
+
+	i = 0;
+	batch_buf[i++] = MI_NOOP;
+	batch_buf[i++] = MI_NOOP;
+	batch_buf[i++] = MI_BATCH_BUFFER_END;
+	batch_buf[i++] = MI_NOOP;
+	igt_assert(i * sizeof(uint32_t) == batch_size);
+
+	handle = gem_create(drm_fd, batch_size);
+	gem_write(drm_fd, handle, 0, batch_buf, batch_size);
+
+	objs[0].handle = handle;
+
+	execbuf.buffers_ptr = (uintptr_t)objs;
+	execbuf.buffer_count = 1;
+	execbuf.batch_len = batch_size;
+	execbuf.flags = I915_EXEC_RENDER;
+	i915_execbuffer2_set_context_id(execbuf, 0);
+
+	disable_all_screens_and_wait(&ms_data);
+	igt_system_suspend_autoresume();
+	igt_assert(wait_for_suspended());
+
+	for (i = 0; i < 20; i++) {
+		do_ioctl(drm_fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &execbuf);
+		igt_assert(wait_for_suspended());
+	}
+
+	gem_close(drm_fd, handle);
+}
+
+static void system_suspend_modeset_subtest(void)
+{
+	disable_all_screens_and_wait(&ms_data);
+	igt_system_suspend_autoresume();
+	igt_assert(wait_for_suspended());
+
+	enable_one_screen_and_wait(&ms_data);
+	disable_all_screens_and_wait(&ms_data);
+}
+
 /* Enable a screen, activate DPMS, then do a modeset. At some point our driver
  * produced WARNs on this case. */
 static void dpms_mode_unset_subtest(enum screen_type type)
@@ -1427,12 +1509,12 @@ static void cursor_subtest(bool dpms)
 	igt_require(default_mode_params);
 	crtc_id = default_mode_params->crtc_id;
 
-	igt_create_fb(drm_fd, 64, 64, DRM_FORMAT_ARGB8888, I915_TILING_NONE,
-		      &cursor_fb1);
-	igt_create_fb(drm_fd, 64, 64, DRM_FORMAT_ARGB8888, I915_TILING_NONE,
-		      &cursor_fb2);
-	igt_create_fb(drm_fd, 64, 64, DRM_FORMAT_ARGB8888, I915_TILING_X,
-		      &cursor_fb3);
+	igt_create_fb(drm_fd, 64, 64, DRM_FORMAT_ARGB8888,
+		      LOCAL_DRM_FORMAT_MOD_NONE, &cursor_fb1);
+	igt_create_fb(drm_fd, 64, 64, DRM_FORMAT_ARGB8888,
+		      LOCAL_DRM_FORMAT_MOD_NONE, &cursor_fb2);
+	igt_create_fb(drm_fd, 64, 64, DRM_FORMAT_ARGB8888,
+		      LOCAL_I915_FORMAT_MOD_X_TILED, &cursor_fb3);
 
 	fill_igt_fb(&cursor_fb1, 0xFF00FFFF);
 	fill_igt_fb(&cursor_fb2, 0xFF00FF00);
@@ -1442,54 +1524,54 @@ static void cursor_subtest(bool dpms)
 
 	rc = drmModeSetCursor(drm_fd, crtc_id, cursor_fb1.gem_handle,
 			      cursor_fb1.width, cursor_fb1.height);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	rc = drmModeMoveCursor(drm_fd, crtc_id, 0, 0);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_active());
 
 	disable_or_dpms_all_screens_and_wait(&ms_data, dpms);
 
 	/* First, just move the cursor. */
 	rc = drmModeMoveCursor(drm_fd, crtc_id, 1, 1);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Then unset it, and set a new one. */
 	rc = drmModeSetCursor(drm_fd, crtc_id, 0, 0, 0);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	rc = drmModeSetCursor(drm_fd, crtc_id, cursor_fb2.gem_handle,
 			      cursor_fb1.width, cursor_fb2.height);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Move the new cursor. */
 	rc = drmModeMoveCursor(drm_fd, crtc_id, 2, 2);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Now set a new one without unsetting the previous one. */
 	rc = drmModeSetCursor(drm_fd, crtc_id, cursor_fb1.gem_handle,
 			      cursor_fb1.width, cursor_fb1.height);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Cursor 3 was created with tiling and painted with a GTT mmap, so
 	 * hopefully it has some fences around it. */
 	rc = drmModeRmFB(drm_fd, cursor_fb3.fb_id);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	gem_set_tiling(drm_fd, cursor_fb3.gem_handle, false, cursor_fb3.stride);
 	igt_assert(wait_for_suspended());
 
 	rc = drmModeSetCursor(drm_fd, crtc_id, cursor_fb3.gem_handle,
 			      cursor_fb3.width, cursor_fb3.height);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Make sure nothing remains for the other tests. */
 	rc = drmModeSetCursor(drm_fd, crtc_id, 0, 0, 0);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 }
 
@@ -1539,7 +1621,7 @@ static void test_one_plane(bool dpms, uint32_t plane_id,
 	uint32_t crtc_id;
 	struct igt_fb plane_fb1, plane_fb2;
 	int32_t crtc_x = 0, crtc_y = 0;
-	unsigned int tiling;
+	uint64_t tiling;
 
 	disable_all_screens_and_wait(&ms_data);
 
@@ -1551,19 +1633,19 @@ static void test_one_plane(bool dpms, uint32_t plane_id,
 		plane_format = DRM_FORMAT_XRGB8888;
 		plane_w = 64;
 		plane_h = 64;
-		tiling = I915_TILING_X;
+		tiling = LOCAL_I915_FORMAT_MOD_X_TILED;
 		break;
 	case PLANE_PRIMARY:
 		plane_format = DRM_FORMAT_XRGB8888;
 		plane_w = default_mode_params->mode->hdisplay;
 		plane_h = default_mode_params->mode->vdisplay;
-		tiling = I915_TILING_X;
+		tiling = LOCAL_I915_FORMAT_MOD_X_TILED;
 		break;
 	case PLANE_CURSOR:
 		plane_format = DRM_FORMAT_ARGB8888;
 		plane_w = 64;
 		plane_h = 64;
-		tiling = I915_TILING_NONE;
+		tiling = LOCAL_DRM_FORMAT_MOD_NONE;
 		break;
 	default:
 		igt_assert(0);
@@ -1583,7 +1665,7 @@ static void test_one_plane(bool dpms, uint32_t plane_id,
 			     0, 0, plane_fb1.width, plane_fb1.height,
 			     0 << 16, 0 << 16, plane_fb1.width << 16,
 			     plane_fb1.height << 16);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 
 	disable_or_dpms_all_screens_and_wait(&ms_data, dpms);
 
@@ -1596,19 +1678,19 @@ static void test_one_plane(bool dpms, uint32_t plane_id,
 			     crtc_x, crtc_y, plane_fb1.width, plane_fb1.height,
 			     0 << 16, 0 << 16, plane_fb1.width << 16,
 			     plane_fb1.height << 16);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Unset, then change the plane. */
 	rc = drmModeSetPlane(drm_fd, plane_id, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	rc = drmModeSetPlane(drm_fd, plane_id, crtc_id, plane_fb2.fb_id, 0,
 			     crtc_x, crtc_y, plane_fb2.width, plane_fb2.height,
 			     0 << 16, 0 << 16, plane_fb2.width << 16,
 			     plane_fb2.height << 16);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Now change the plane without unsetting first. */
@@ -1616,12 +1698,12 @@ static void test_one_plane(bool dpms, uint32_t plane_id,
 			     crtc_x, crtc_y, plane_fb1.width, plane_fb1.height,
 			     0 << 16, 0 << 16, plane_fb1.width << 16,
 			     plane_fb1.height << 16);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 
 	/* Make sure nothing remains for the other tests. */
 	rc = drmModeSetPlane(drm_fd, plane_id, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
-	igt_assert(rc == 0);
+	igt_assert_eq(rc, 0);
 	igt_assert(wait_for_suspended());
 }
 
@@ -1660,11 +1742,11 @@ static void planes_subtest(bool universal, bool dpms)
 
 	if (universal) {
 		rc = drmSetClientCap(drm_fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 0);
-		igt_assert(rc == 0);
+		igt_assert_eq(rc, 0);
 
-		igt_assert(planes_tested >= 3);
+		igt_assert_lte(3, planes_tested);
 	} else {
-		igt_assert(planes_tested >= 1);
+		igt_assert_lte(1, planes_tested);
 	}
 }
 
@@ -1682,7 +1764,8 @@ static void fences_subtest(bool dpms)
 	params.connector_id = default_mode_params->connector_id;
 	params.mode = default_mode_params->mode;
 	igt_create_fb(drm_fd, params.mode->hdisplay, params.mode->vdisplay,
-		      DRM_FORMAT_XRGB8888, I915_TILING_X, &params.fb);
+		      DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_X_TILED,
+		      &params.fb);
 
 	/* Even though we passed "true" as the tiling argument, double-check
 	 * that the fb is really tiled. */
@@ -1723,7 +1806,7 @@ static void fences_subtest(bool dpms)
 	igt_assert(munmap(buf_ptr, params.fb.size) == 0);
 }
 
-int rounds = 50;
+int rounds = 40;
 bool stay = false;
 
 static int opt_handler(int opt, int opt_index)
@@ -1753,7 +1836,7 @@ int main(int argc, char *argv[])
 		{ 0, 0, 0, 0 }
 	};
 
-	igt_subtest_init_parse_opts(argc, argv, "", long_options,
+	igt_subtest_init_parse_opts(&argc, argv, "", long_options,
 				    help_str, opt_handler);
 
 	/* Skip instead of failing in case the machine is not prepared to reach
@@ -1795,6 +1878,8 @@ int main(int argc, char *argv[])
 		gem_execbuf_subtest();
 	igt_subtest("gem-idle")
 		gem_idle_subtest();
+	igt_subtest("gem-evict-pwrite")
+		gem_evict_pwrite_subtest();
 
 	/* Planes and cursors */
 	igt_subtest("cursor")
@@ -1850,6 +1935,10 @@ int main(int argc, char *argv[])
 	/* System suspend */
 	igt_subtest("system-suspend")
 		system_suspend_subtest();
+	igt_subtest("system-suspend-execbuf")
+		system_suspend_execbuf_subtest();
+	igt_subtest("system-suspend-modeset")
+		system_suspend_modeset_subtest();
 
 	/* GEM stress */
 	igt_subtest("gem-execbuf-stress")
